@@ -27,11 +27,14 @@
 #include <gst/gstminiobject.h>
 #include <gst/gstclock.h>
 #include <gst/gstcaps.h>
+#include <gst/gstmemory.h>
 
 G_BEGIN_DECLS
 
+extern GType _gst_buffer_type;
+
 typedef struct _GstBuffer GstBuffer;
-typedef struct _GstBufferClass GstBufferClass;
+typedef struct _GstBufferPool GstBufferPool;
 
 /**
  * GST_BUFFER_TRACE_NAME:
@@ -40,13 +43,10 @@ typedef struct _GstBufferClass GstBufferClass;
  */
 #define GST_BUFFER_TRACE_NAME           "GstBuffer"
 
-#define GST_TYPE_BUFFER                         (gst_buffer_get_type())
-#define GST_IS_BUFFER(obj)                      (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_BUFFER))
-#define GST_IS_BUFFER_CLASS(klass)              (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_BUFFER))
-#define GST_BUFFER_GET_CLASS(obj)               (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_BUFFER, GstBufferClass))
-#define GST_BUFFER(obj)                         (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_BUFFER, GstBuffer))
-#define GST_BUFFER_CLASS(klass)                 (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_BUFFER, GstBufferClass))
+#define GST_TYPE_BUFFER                         (_gst_buffer_type)
+#define GST_IS_BUFFER(obj)                      (GST_IS_MINI_OBJECT_TYPE(obj, GST_TYPE_BUFFER))
 #define GST_BUFFER_CAST(obj)                    ((GstBuffer *)(obj))
+#define GST_BUFFER(obj)                         (GST_BUFFER_CAST(obj))
 
 /**
  * GST_BUFFER_FLAGS:
@@ -80,20 +80,6 @@ typedef struct _GstBufferClass GstBufferClass;
  */
 #define GST_BUFFER_FLAG_UNSET(buf,flag)         GST_MINI_OBJECT_FLAG_UNSET (buf, flag)
 
-/**
- * GST_BUFFER_DATA:
- * @buf: a #GstBuffer.
- *
- * A pointer to the data element of this buffer.
- */
-#define GST_BUFFER_DATA(buf)                    (GST_BUFFER_CAST(buf)->data)
-/**
- * GST_BUFFER_SIZE:
- * @buf: a #GstBuffer.
- *
- * The size in bytes of the data in this buffer.
- */
-#define GST_BUFFER_SIZE(buf)                    (GST_BUFFER_CAST(buf)->size)
 /**
  * GST_BUFFER_TIMESTAMP:
  * @buf: a #GstBuffer.:
@@ -111,13 +97,6 @@ typedef struct _GstBufferClass GstBufferClass;
  * Value will be %GST_CLOCK_TIME_NONE if the duration is unknown.
  */
 #define GST_BUFFER_DURATION(buf)                (GST_BUFFER_CAST(buf)->duration)
-/**
- * GST_BUFFER_CAPS:
- * @buf: a #GstBuffer.
- *
- * The caps for this buffer.
- */
-#define GST_BUFFER_CAPS(buf)                    (GST_BUFFER_CAST(buf)->caps)
 /**
  * GST_BUFFER_OFFSET:
  * @buf: a #GstBuffer.
@@ -132,28 +111,6 @@ typedef struct _GstBufferClass GstBufferClass;
  * The offset in the source file of the end of this buffer.
  */
 #define GST_BUFFER_OFFSET_END(buf)              (GST_BUFFER_CAST(buf)->offset_end)
-/**
- * GST_BUFFER_MALLOCDATA:
- * @buf: a #GstBuffer.
- *
- * A pointer to any data allocated for this buffer using g_malloc(). If this is
- * non-NULL, this memory will be freed at the end of the buffer's lifecycle
- * (i.e. when its refcount becomes zero).
- */
-#define GST_BUFFER_MALLOCDATA(buf)              (GST_BUFFER_CAST(buf)->malloc_data)
-/**
- * GST_BUFFER_FREE_FUNC:
- * @buf: a #GstBuffer.
- *
- * A pointer to a function that will be called on the buffer's malloc_data when
- * this buffer is finalized. Defaults to g_free().
- *
- * Note that the free function only affects the buffer's malloc_data; if the
- * buffer's malloc_data is %NULL, the function will not be called.
- *
- * Since: 0.10.22
- */
-#define GST_BUFFER_FREE_FUNC(buf)               (GST_BUFFER_CAST(buf)->free_func)
 
 /**
  * GST_BUFFER_OFFSET_NONE:
@@ -201,49 +158,61 @@ typedef struct _GstBufferClass GstBufferClass;
 #define GST_BUFFER_IS_DISCONT(buffer)   (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT))
 
 /**
- * GstBufferFlag:
- * @GST_BUFFER_FLAG_READONLY: the buffer is read-only. This means the data of
- * the buffer should not be modified. The metadata might still be modified.
- * @GST_BUFFER_FLAG_PREROLL: the buffer is part of a preroll and should not be
- * displayed.
- * @GST_BUFFER_FLAG_DISCONT: the buffer marks a discontinuity in the stream.
- * This typically occurs after a seek or a dropped buffer from a live or
- * network source.
- * @GST_BUFFER_FLAG_IN_CAPS: the buffer has been added as a field in a #GstCaps.
- * @GST_BUFFER_FLAG_GAP: the buffer has been created to fill a gap in the
- * stream and contains media neutral data (elements can switch to optimized code
- * path that ignores the buffer content).
- * @GST_BUFFER_FLAG_DELTA_UNIT: this unit cannot be decoded independently.
- * @GST_BUFFER_FLAG_MEDIA1: a flag whose use is specific to the caps of the buffer. Since: 0.10.23.
- * @GST_BUFFER_FLAG_MEDIA2: a flag whose use is specific to the caps of the buffer. Since: 0.10.23.
- * @GST_BUFFER_FLAG_MEDIA3: a flag whose use is specific to the caps of the buffer. Since: 0.10.23.
- * @GST_BUFFER_FLAG_LAST: additional flags can be added starting from this flag.
+ * GstBufferFlags:
+ * @GST_BUFFER_FLAG_LIVE:        the buffer is live data and should be discarded in
+ *                               the PAUSED state.
+ * @GST_BUFFER_FLAG_DECODE_ONLY: the buffer contains data that should be dropped
+ *                               because it will be clipped against the segment
+ *                               boundaries or because it does not contain data
+ *                               that should be shown to the user.
+ * @GST_BUFFER_FLAG_DISCONT:     the buffer marks a data discontinuity in the stream.
+ *                               This typically occurs after a seek or a dropped buffer
+ *                               from a live or network source.
+ * @GST_BUFFER_FLAG_RESYNC:      the buffer timestamp might have a discontinuity
+ *                               and this buffer is a good point to resynchronize.
+ * @GST_BUFFER_FLAG_CORRUPTED:   the buffer data is corrupted.
+ * @GST_BUFFER_FLAG_MARKER:      the buffer contains a media specific marker. for
+ *                               video this is typically the end of a frame boundary, for audio
+ *                               this is usually the end of a talkspurt.
+ * @GST_BUFFER_FLAG_HEADER:      the buffer contains header information that is
+ *                               needed to decode the following data
+ * @GST_BUFFER_FLAG_GAP:         the buffer has been created to fill a gap in the
+ *                               stream and contains media neutral data (elements can
+ *                               switch to optimized code path that ignores the buffer
+ *                               content).
+ * @GST_BUFFER_FLAG_DROPPABLE:   the buffer can be dropped without breaking the
+ *                               stream, for example to reduce bandwidth.
+ * @GST_BUFFER_FLAG_DELTA_UNIT:  this unit cannot be decoded independently.
+ * @GST_BUFFER_FLAG_IN_CAPS:     the buffer has been added as a field in a #GstCaps.
+ * @GST_BUFFER_FLAG_LAST:        additional media specific flags can be added starting from
+ *                               this flag.
  *
  * A set of buffer flags used to describe properties of a #GstBuffer.
  */
 typedef enum {
-  GST_BUFFER_FLAG_READONLY   = GST_MINI_OBJECT_FLAG_READONLY,
-  GST_BUFFER_FLAG_PREROLL    = (GST_MINI_OBJECT_FLAG_LAST << 0),
-  GST_BUFFER_FLAG_DISCONT    = (GST_MINI_OBJECT_FLAG_LAST << 1),
-  GST_BUFFER_FLAG_IN_CAPS    = (GST_MINI_OBJECT_FLAG_LAST << 2),
-  GST_BUFFER_FLAG_GAP        = (GST_MINI_OBJECT_FLAG_LAST << 3),
-  GST_BUFFER_FLAG_DELTA_UNIT = (GST_MINI_OBJECT_FLAG_LAST << 4),
-  GST_BUFFER_FLAG_MEDIA1     = (GST_MINI_OBJECT_FLAG_LAST << 5),
-  GST_BUFFER_FLAG_MEDIA2     = (GST_MINI_OBJECT_FLAG_LAST << 6),
-  GST_BUFFER_FLAG_MEDIA3     = (GST_MINI_OBJECT_FLAG_LAST << 7),
-  GST_BUFFER_FLAG_LAST       = (GST_MINI_OBJECT_FLAG_LAST << 8)
-} GstBufferFlag;
+  GST_BUFFER_FLAG_LIVE        = (GST_MINI_OBJECT_FLAG_LAST << 0),
+  GST_BUFFER_FLAG_DECODE_ONLY = (GST_MINI_OBJECT_FLAG_LAST << 1),
+  GST_BUFFER_FLAG_DISCONT     = (GST_MINI_OBJECT_FLAG_LAST << 2),
+  GST_BUFFER_FLAG_RESYNC      = (GST_MINI_OBJECT_FLAG_LAST << 3),
+  GST_BUFFER_FLAG_CORRUPTED   = (GST_MINI_OBJECT_FLAG_LAST << 4),
+  GST_BUFFER_FLAG_MARKER      = (GST_MINI_OBJECT_FLAG_LAST << 5),
+  GST_BUFFER_FLAG_HEADER      = (GST_MINI_OBJECT_FLAG_LAST << 6),
+  GST_BUFFER_FLAG_GAP         = (GST_MINI_OBJECT_FLAG_LAST << 7),
+  GST_BUFFER_FLAG_DROPPABLE   = (GST_MINI_OBJECT_FLAG_LAST << 8),
+  GST_BUFFER_FLAG_DELTA_UNIT  = (GST_MINI_OBJECT_FLAG_LAST << 9),
+  GST_BUFFER_FLAG_IN_CAPS     = (GST_MINI_OBJECT_FLAG_LAST << 10),
+
+  GST_BUFFER_FLAG_LAST        = (GST_MINI_OBJECT_FLAG_LAST << 20)
+} GstBufferFlags;
 
 /**
  * GstBuffer:
  * @mini_object: the parent structure
- * @data: pointer to the buffer data
- * @size: size of buffer data
+ * @pool: pointer to the pool owner of the buffer
  * @timestamp: timestamp of the buffer, can be #GST_CLOCK_TIME_NONE when the
  *     timestamp is not known or relevant.
  * @duration: duration in time of the buffer data, can be #GST_CLOCK_TIME_NONE
  *     when the duration is not known or relevant.
- * @caps: the #GstCaps describing the data format in this buffer
  * @offset: a media specific offset for the buffer data.
  *     For video frames, this is the frame number of this buffer.
  *     For audio samples, this is the offset of the first sample in this buffer.
@@ -251,11 +220,6 @@ typedef enum {
  *       byte in this buffer.
  * @offset_end: the last offset contained in this buffer. It has the same
  *     format as @offset.
- * @malloc_data: a pointer to the allocated memory associated with this buffer.
- *     When the buffer is freed, this data will freed with @free_func.
- * @free_func: a custom function that will be called with @malloc_data, defaults
- *     to g_free(). Since 0.10.22.
- * @parent: the parent buffer if this is a subbuffer. Since 0.10.26.
  *
  * The structure of a #GstBuffer. Use the associated macros to access the public
  * variables.
@@ -264,60 +228,73 @@ struct _GstBuffer {
   GstMiniObject          mini_object;
 
   /*< public >*/ /* with COW */
-  /* pointer to data and its size */
-  guint8                *data;
-  guint                  size;
+  GstBufferPool         *pool;
 
   /* timestamp */
   GstClockTime           timestamp;
   GstClockTime           duration;
 
-  /* the media type of this buffer */
-  GstCaps               *caps;
-
   /* media specific offset */
   guint64                offset;
   guint64                offset_end;
-
-  guint8                *malloc_data;
-
-  /* ABI Added */
-  GFreeFunc              free_func;
-  GstBuffer             *parent;
-
-  /*< private >*/
-  gpointer _gst_reserved[GST_PADDING - 2];
 };
 
-struct _GstBufferClass {
-  GstMiniObjectClass    mini_object_class;
-};
-
-GType       gst_buffer_get_type (void);
+GType       gst_buffer_get_type            (void);
 
 /* allocation */
-GstBuffer * gst_buffer_new               (void);
-GstBuffer * gst_buffer_new_and_alloc     (guint size);
-GstBuffer * gst_buffer_try_new_and_alloc (guint size);
+GstBuffer * gst_buffer_new                 (void);
+GstBuffer * gst_buffer_new_allocate        (const GstAllocator * allocator, gsize size, gsize align);
+GstBuffer * gst_buffer_new_wrapped_full    (gpointer data, GFreeFunc free_func, gsize offset, gsize size);
+GstBuffer * gst_buffer_new_wrapped         (gpointer data, gsize size);
+
+/* memory blocks */
+guint       gst_buffer_n_memory            (GstBuffer *buffer);
+void        gst_buffer_take_memory         (GstBuffer *buffer, gint idx, GstMemory *mem);
+GstMemory * gst_buffer_peek_memory         (GstBuffer *buffer, guint idx, GstMapFlags flags);
+void        gst_buffer_remove_memory_range (GstBuffer *buffer, guint idx, guint length);
 
 /**
- * gst_buffer_set_data:
- * @buf: a #GstBuffer
- * @data: The data (a #guint8 *) to set on the buffer.
- * @size: The size (in bytes, as a #guint) of the data being set.
+ * gst_buffer_remove_memory:
+ * @b: a #GstBuffer.
+ * @i: an index
  *
- * A convenience function to set the data and size on a buffer.
- * This will replace any existing data pointer set on this buffer, but will
- * not change GST_BUFFER_MALLOCDATA(), if any. Callers should ensure that
- * GST_BUFFER_MALLOCDATA() is non-NULL, or should free that and set it to NULL.
- *
- * No checks are done on the data or size arguments passed.
+ * Remove the memory block in @b at @i.
  */
-#define         gst_buffer_set_data(buf, data, size)    \
-G_STMT_START {                                          \
-  GST_BUFFER_DATA (buf) = data;                         \
-  GST_BUFFER_SIZE (buf) = size;                         \
-} G_STMT_END
+#define     gst_buffer_remove_memory(b,i)  gst_buffer_remove_memory_range ((b), (i), 1)
+
+gsize       gst_buffer_fill                (GstBuffer *buffer, gsize offset,
+                                            gconstpointer src, gsize size);
+gsize       gst_buffer_extract             (GstBuffer *buffer, gsize offset,
+                                            gpointer dest, gsize size);
+gint        gst_buffer_memcmp              (GstBuffer *buffer, gsize offset,
+                                            gconstpointer mem, gsize size);
+gsize       gst_buffer_memset              (GstBuffer *buffer, gsize offset,
+                                            guint8 val, gsize size);
+
+gsize       gst_buffer_get_sizes           (GstBuffer *buffer, gsize *offset, gsize *maxsize);
+void        gst_buffer_resize              (GstBuffer *buffer, gssize offset, gsize size);
+
+/**
+ * gst_buffer_get_size:
+ * @b: a #GstBuffer.
+ *
+ * Get the size of @b.
+ */
+#define     gst_buffer_get_size(b)         gst_buffer_get_sizes ((b), NULL, NULL)
+/**
+ * gst_buffer_set_size:
+ * @b: a #GstBuffer.
+ * @s: a new size
+ *
+ * Set the size of @b to @s. This will remove or trim the memory blocks
+ * in the buffer.
+ */
+#define     gst_buffer_set_size(b,s)       gst_buffer_resize ((b), 0, (s))
+
+/* getting memory */
+gpointer    gst_buffer_map                 (GstBuffer *buffer, gsize *size, gsize *maxsize,
+                                            GstMapFlags flags);
+gboolean    gst_buffer_unmap               (GstBuffer *buffer, gpointer data, gsize size);
 
 /* refcounting */
 /**
@@ -386,35 +363,46 @@ gst_buffer_copy (const GstBuffer * buf)
 
 /**
  * GstBufferCopyFlags:
+ * @GST_BUFFER_COPY_NONE: copy nothing
  * @GST_BUFFER_COPY_FLAGS: flag indicating that buffer flags should be copied
  * @GST_BUFFER_COPY_TIMESTAMPS: flag indicating that buffer timestamp, duration,
  * offset and offset_end should be copied
- * @GST_BUFFER_COPY_CAPS: flag indicating that buffer caps should be copied
+ * @GST_BUFFER_COPY_MEMORY: flag indicating that buffer memory should be copied
+ * and appended to already existing memory
+ * @GST_BUFFER_COPY_MERGE: flag indicating that buffer memory should be
+ * merged
  *
- * A set of flags that can be provided to the gst_buffer_copy_metadata()
- * function to specify which metadata fields should be copied.
- *
- * Since: 0.10.13
+ * A set of flags that can be provided to the gst_buffer_copy_into()
+ * function to specify which items should be copied.
  */
 typedef enum {
-  GST_BUFFER_COPY_FLAGS      = (1 << 0),
-  GST_BUFFER_COPY_TIMESTAMPS = (1 << 1),
-  GST_BUFFER_COPY_CAPS       = (1 << 2)
+  GST_BUFFER_COPY_NONE           = 0,
+  GST_BUFFER_COPY_FLAGS          = (1 << 0),
+  GST_BUFFER_COPY_TIMESTAMPS     = (1 << 1),
+  GST_BUFFER_COPY_MEMORY         = (1 << 2),
+  GST_BUFFER_COPY_MERGE          = (1 << 3)
 } GstBufferCopyFlags;
 
+/**
+ * GST_BUFFER_COPY_METADATA:
+ *
+ * Combination of all possible metadata fields that can be copied with
+ * gst_buffer_copy_into().
+ */
+#define GST_BUFFER_COPY_METADATA       (GST_BUFFER_COPY_FLAGS | GST_BUFFER_COPY_TIMESTAMPS)
+
 /**
  * GST_BUFFER_COPY_ALL:
  *
  * Combination of all possible fields that can be copied with
- * gst_buffer_copy_metadata().
- *
- * Since: 0.10.13
+ * gst_buffer_copy_into().
  */
-#define GST_BUFFER_COPY_ALL (GST_BUFFER_COPY_FLAGS | GST_BUFFER_COPY_TIMESTAMPS | GST_BUFFER_COPY_CAPS)
+#define GST_BUFFER_COPY_ALL  ((GstBufferCopyFlags)(GST_BUFFER_COPY_METADATA | GST_BUFFER_COPY_MEMORY))
 
-/* copies metadata into newly allocated buffer */
-void            gst_buffer_copy_metadata        (GstBuffer *dest, const GstBuffer *src,
-                                                 GstBufferCopyFlags flags);
+/* copies memory or metadata into newly allocated buffer */
+void            gst_buffer_copy_into            (GstBuffer *dest, GstBuffer *src,
+                                                 GstBufferCopyFlags flags,
+                                                 gsize offset, gsize size);
 
 /**
  * gst_buffer_is_writable:
@@ -439,11 +427,6 @@ void            gst_buffer_copy_metadata        (GstBuffer *dest, const GstBuffe
  */
 #define         gst_buffer_make_writable(buf)   GST_BUFFER_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (buf)))
 
-/* Ensure that the metadata of the buffer is writable, even if the buffer data
- * isn't */
-gboolean        gst_buffer_is_metadata_writable (GstBuffer *buf);
-GstBuffer*      gst_buffer_make_metadata_writable (GstBuffer *buf);
-
 /**
  * gst_buffer_replace:
  * @obuf: (inout) (transfer full): pointer to a pointer to a #GstBuffer to be
@@ -465,15 +448,23 @@ G_STMT_START {                                                                \
       GST_MINI_OBJECT_CAST (nbuf));                       \
 } G_STMT_END
 
-GstCaps*        gst_buffer_get_caps             (GstBuffer *buffer);
-void            gst_buffer_set_caps             (GstBuffer *buffer, GstCaps *caps);
-
-/* creating a subbuffer */
-GstBuffer*      gst_buffer_create_sub           (GstBuffer *parent, guint offset, guint size);
+/* creating a region */
+GstBuffer*      gst_buffer_copy_region          (GstBuffer *parent, GstBufferCopyFlags flags,
+                                                 gsize offset, gsize size);
 
 /* span, two buffers, intelligently */
 gboolean        gst_buffer_is_span_fast         (GstBuffer *buf1, GstBuffer *buf2);
-GstBuffer*      gst_buffer_span                 (GstBuffer *buf1, guint32 offset, GstBuffer *buf2, guint32 len);
+GstBuffer*      gst_buffer_span                 (GstBuffer *buf1, gsize offset, GstBuffer *buf2, gsize size);
+
+/* metadata */
+#include <gst/gstmeta.h>
+
+GstMeta *       gst_buffer_get_meta             (GstBuffer *buffer, const GstMetaInfo *info);
+GstMeta *       gst_buffer_add_meta             (GstBuffer *buffer, const GstMetaInfo *info,
+                                                 gpointer params);
+gboolean        gst_buffer_remove_meta          (GstBuffer *buffer, GstMeta *meta);
+
+GstMeta *       gst_buffer_iterate_meta         (GstBuffer *buffer, gpointer *state);
 
 /**
  * gst_value_set_buffer:
@@ -482,7 +473,7 @@ GstBuffer*      gst_buffer_span                 (GstBuffer *buf1, guint32 offset
  *
  * Sets @b as the value of @v.  Caller retains reference to buffer.
  */
-#define         gst_value_set_buffer(v,b)       gst_value_set_mini_object(v, GST_MINI_OBJECT_CAST(b))
+#define         gst_value_set_buffer(v,b)       g_value_set_boxed((v),(b))
 /**
  * gst_value_take_buffer:
  * @v: a #GValue to receive the data
@@ -490,7 +481,7 @@ GstBuffer*      gst_buffer_span                 (GstBuffer *buf1, guint32 offset
  *
  * Sets @b as the value of @v.  Caller gives away reference to buffer.
  */
-#define         gst_value_take_buffer(v,b)      gst_value_take_mini_object(v, GST_MINI_OBJECT_CAST(b))
+#define         gst_value_take_buffer(v,b)      g_value_take_boxed(v,(b))
 /**
  * gst_value_get_buffer:
  * @v: a #GValue to query
@@ -501,7 +492,7 @@ GstBuffer*      gst_buffer_span                 (GstBuffer *buf1, guint32 offset
  *
  * Returns: (transfer none): buffer
  */
-#define         gst_value_get_buffer(v)         GST_BUFFER_CAST (gst_value_get_mini_object(v))
+#define         gst_value_get_buffer(v)         GST_BUFFER_CAST (g_value_get_boxed(v))
 
 G_END_DECLS
 
