@@ -25,7 +25,6 @@
 G_BEGIN_DECLS
 
 typedef struct _GstMessage GstMessage;
-typedef struct _GstMessageClass GstMessageClass;
 
 /**
  * GstMessageType:
@@ -74,11 +73,11 @@ typedef struct _GstMessageClass GstMessageClass;
  * @GST_MESSAGE_SEGMENT_START posted a GST_MESSAGE_SEGMENT_DONE message.
  * @GST_MESSAGE_DURATION: The duration of a pipeline changed. The application
  * can get the new duration with a duration query.
- * @GST_MESSAGE_ASYNC_START: Posted by elements when they start an ASYNC state
- * change. This message is not forwarded to the application but is used
+ * @GST_MESSAGE_ASYNC_START: Posted by elements when they start an ASYNC
+ * #GstStateChange. This message is not forwarded to the application but is used
  * internally. Since: 0.10.13.
- * @GST_MESSAGE_ASYNC_DONE: Posted by elements when they complete an ASYNC state
- * change. The application will only receive this message from the toplevel
+ * @GST_MESSAGE_ASYNC_DONE: Posted by elements when they complete an ASYNC
+ * #GstStateChange. The application will only receive this message from the toplevel
  * pipeline. Since: 0.10.13
  * @GST_MESSAGE_LATENCY: Posted by elements when their latency changes. The
  * application should recalculate and distribute a new latency. Since: 0.10.12
@@ -88,6 +87,7 @@ typedef struct _GstMessageClass GstMessageClass;
  * @GST_MESSAGE_STEP_START: A stepping operation was started. Since: 0.10.24
  * @GST_MESSAGE_QOS: A buffer was dropped or an element changed its processing
  * strategy for Quality of Service reasons. Since: 0.10.29
+ * @GST_MESSAGE_PROGRESS: A progress message. Since: 0.10.33
  * @GST_MESSAGE_ANY: mask for all of the above messages.
  *
  * The different message types that are available.
@@ -123,6 +123,7 @@ typedef enum
   GST_MESSAGE_REQUEST_STATE     = (1 << 22),
   GST_MESSAGE_STEP_START        = (1 << 23),
   GST_MESSAGE_QOS               = (1 << 24),
+  GST_MESSAGE_PROGRESS          = (1 << 25),
   GST_MESSAGE_ANY               = ~0
 } GstMessageType;
 
@@ -131,6 +132,7 @@ typedef enum
 #include <gst/gstelement.h>
 #include <gst/gsttaglist.h>
 #include <gst/gststructure.h>
+#include <gst/gstquery.h>
 
 /**
  * GST_MESSAGE_TRACE_NAME:
@@ -140,12 +142,9 @@ typedef enum
 #define GST_MESSAGE_TRACE_NAME  "GstMessage"
 
 #define GST_TYPE_MESSAGE                         (gst_message_get_type())
-#define GST_IS_MESSAGE(obj)                      (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_MESSAGE))
-#define GST_IS_MESSAGE_CLASS(klass)              (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_MESSAGE))
-#define GST_MESSAGE_GET_CLASS(obj)               (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_MESSAGE, GstMessageClass))
-#define GST_MESSAGE(obj)                         (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_MESSAGE, GstMessage))
-#define GST_MESSAGE_CLASS(klass)                 (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_MESSAGE, GstMessageClass))
+#define GST_IS_MESSAGE(obj)                      (GST_IS_MINI_OBJECT_TYPE (obj, GST_TYPE_MESSAGE))
 #define GST_MESSAGE_CAST(obj)                    ((GstMessage*)(obj))
+#define GST_MESSAGE(obj)                         (GST_MESSAGE_CAST(obj))
 
 /* the lock is used to handle the synchronous handling of messages,
  * the emiting thread is block until the handling thread processed
@@ -188,6 +187,15 @@ typedef enum
  * Get the object that posted @message.
  */
 #define GST_MESSAGE_SRC(message)        (GST_MESSAGE_CAST(message)->src)
+
+/**
+ * GST_MESSAGE_SEQNUM:
+ * @message: a #GstMessage
+ *
+ * Get the sequence number of @message.
+ */
+#define GST_MESSAGE_SEQNUM(message)     (GST_MESSAGE_CAST(message)->seqnum)
+
 /**
  * GST_MESSAGE_SRC_NAME:
  * @message: a #GstMessage
@@ -240,13 +248,35 @@ typedef enum {
   GST_STREAM_STATUS_TYPE_STOP     = 10
 } GstStreamStatusType;
 
+/**
+ * GstProgressType:
+ * @GST_PROGRESS_TYPE_START: A new task started.
+ * @GST_PROGRESS_TYPE_CONTINUE: A task completed and a new one continues.
+ * @GST_PROGRESS_TYPE_COMPLETE: A task completed.
+ * @GST_PROGRESS_TYPE_CANCELED: A task was canceled.
+ * @GST_PROGRESS_TYPE_ERROR: A task caused an error. An error message is also
+ *          posted on the bus.
+ *
+ * The type of a %GST_MESSAGE_PROGRESS. The progress messages inform the
+ * application of the status of assynchronous tasks.
+ *
+ * Since: 0.10.33
+ */
+typedef enum {
+  GST_PROGRESS_TYPE_START    = 0,
+  GST_PROGRESS_TYPE_CONTINUE = 1,
+  GST_PROGRESS_TYPE_COMPLETE = 2,
+  GST_PROGRESS_TYPE_CANCELED = 3,
+  GST_PROGRESS_TYPE_ERROR    = 4,
+} GstProgressType;
+
 /**
  * GstMessage:
  * @mini_object: the parent structure
  * @type: the #GstMessageType of the message
  * @timestamp: the timestamp of the message
  * @src: the src of the message
- * @structure: the #GstStructure containing the message info.
+ * @seqnum: the sequence number of the message
  *
  * A #GstMessage.
  */
@@ -254,32 +284,15 @@ struct _GstMessage
 {
   GstMiniObject mini_object;
 
-  /*< private >*//* with MESSAGE_LOCK */
-  GMutex *lock;                 /* lock and cond for async delivery */
-  GCond *cond;
-
   /*< public > *//* with COW */
   GstMessageType type;
   guint64 timestamp;
   GstObject *src;
+  guint32 seqnum;
 
-  GstStructure *structure;
-
-  /*< private >*/
-  union {
-    struct {
-      guint32 seqnum;
-    } ABI;
-    /* + 0 to mark ABI change for future greppage */
-    gpointer _gst_reserved[GST_PADDING + 0];
-  } abidata;
-};
-
-struct _GstMessageClass {
-  GstMiniObjectClass mini_object_class;
-
-  /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  /*< private >*//* with MESSAGE_LOCK */
+  GMutex *lock;                 /* lock and cond for async delivery */
+  GCond *cond;
 };
 
 GType           gst_message_get_type            (void);
@@ -330,7 +343,7 @@ gst_message_unref (GstMessage * msg)
  *
  * Creates a copy of the message. Returns a copy of the message.
  *
- * Returns: a new copy of @msg.
+ * Returns: (transfer full): a new copy of @msg.
  *
  * MT safe
  */
@@ -341,19 +354,55 @@ G_INLINE_FUNC GstMessage * gst_message_copy (const GstMessage * msg);
 static inline GstMessage *
 gst_message_copy (const GstMessage * msg)
 {
-  return GST_MESSAGE_CAST (gst_mini_object_copy (GST_MINI_OBJECT_CAST (msg)));
+  return GST_MESSAGE_CAST (gst_mini_object_copy (GST_MINI_OBJECT_CONST_CAST (msg)));
 }
 
+/**
+ * gst_message_is_writable:
+ * @msg: a #GstMessage
+ *
+ * Tests if you can safely write into a message's structure or validly
+ * modify the seqnum and timestamp fields.
+ */
+#define         gst_message_is_writable(msg)     gst_mini_object_is_writable (GST_MINI_OBJECT_CAST (msg))
 /**
  * gst_message_make_writable:
- * @msg: the message to make writable
+ * @msg: (transfer full): the message to make writable
  *
  * Checks if a message is writable. If not, a writable copy is made and
- * returned. Returns a message (possibly a duplicate) that is writable.
+ * returned.
+ *
+ * Returns: (transfer full): a message (possibly a duplicate) that is writable.
  *
  * MT safe
  */
 #define         gst_message_make_writable(msg)  GST_MESSAGE_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (msg)))
+/**
+ * gst_message_replace:
+ * @old_message: (inout) (transfer full): pointer to a pointer to a #GstMessage
+ *     to be replaced.
+ * @new_message: (allow-none) (transfer none): pointer to a #GstMessage that will
+ *     replace the message pointed to by @old_message.
+ *
+ * Modifies a pointer to a #GstMessage to point to a different #GstMessage. The
+ * modification is done atomically (so this is useful for ensuring thread safety
+ * in some cases), and the reference counts are updated appropriately (the old
+ * message is unreffed, the new one is reffed).
+ *
+ * Either @new_message or the #GstMessage pointed to by @old_message may be NULL.
+ */
+#define         gst_message_replace(old_message,new_message) \
+    gst_mini_object_replace ((GstMiniObject **)(old_message), GST_MINI_OBJECT_CAST (new_message))
+
+
+/* custom messages */
+GstMessage *    gst_message_new_custom          (GstMessageType type,
+                                                 GstObject    * src,
+                                                 GstStructure * structure);
+const GstStructure *
+                gst_message_get_structure       (GstMessage *message);
+
+gboolean        gst_message_has_name            (GstMessage *message, const gchar *name);
 
 /* identifiers for events and messages */
 guint32         gst_message_get_seqnum          (GstMessage *message);
@@ -377,9 +426,7 @@ void            gst_message_parse_info          (GstMessage *message, GError **g
 
 /* TAG */
 GstMessage *    gst_message_new_tag             (GstObject * src, GstTagList * tag_list);
-GstMessage *    gst_message_new_tag_full        (GstObject * src, GstPad *pad, GstTagList * tag_list);
 void            gst_message_parse_tag           (GstMessage *message, GstTagList **tag_list);
-void            gst_message_parse_tag_full      (GstMessage *message, GstPad **pad, GstTagList **tag_list);
 
 /* BUFFERING */
 GstMessage *    gst_message_new_buffering         (GstObject * src, gint percent);
@@ -445,11 +492,11 @@ void            gst_message_parse_duration      (GstMessage *message, GstFormat
 GstMessage *    gst_message_new_latency         (GstObject * src);
 
 /* ASYNC_START */
-GstMessage *    gst_message_new_async_start     (GstObject * src, gboolean new_base_time);
-void            gst_message_parse_async_start   (GstMessage *message, gboolean *new_base_time);
+GstMessage *    gst_message_new_async_start     (GstObject * src);
 
 /* ASYNC_DONE */
-GstMessage *    gst_message_new_async_done      (GstObject * src);
+GstMessage *    gst_message_new_async_done      (GstObject * src, gboolean reset_time);
+void            gst_message_parse_async_done    (GstMessage *message, gboolean *reset_time);
 
 /* STRUCTURE CHANGE */
 GstMessage *    gst_message_new_structure_change   (GstObject * src, GstStructureChangeType type,
@@ -490,12 +537,12 @@ void            gst_message_parse_qos_values    (GstMessage * message, gint64 *
                                                  gint * quality);
 void            gst_message_parse_qos_stats     (GstMessage * message, GstFormat * format, guint64 * processed,
                                                  guint64 * dropped);
+/* PROGRESS */
+GstMessage *    gst_message_new_progress           (GstObject * src, GstProgressType type, const gchar *code,
+                                                    const gchar *text);
+void            gst_message_parse_progress         (GstMessage * message, GstProgressType * type, gchar ** code,
+                                                    gchar ** text);
 
-/* custom messages */
-GstMessage *    gst_message_new_custom          (GstMessageType type,
-                                                 GstObject    * src,
-                                                 GstStructure * structure);
-const GstStructure *  gst_message_get_structure (GstMessage *message);
 
 G_END_DECLS
 
