@@ -53,20 +53,17 @@
  *     controller = gst_object_control_properties (object, "prop1", "prop2",...);
  *   </para></listitem>
  *   <listitem><para>
- *     create a #GstControlSource.
+ *     Get a #GstControlSource for the property and set it up.
  *     csource = gst_interpolation_control_source_new ();
  *     gst_interpolation_control_source_set_interpolation_mode(csource, mode);
- *   </para></listitem>
- *   <listitem><para>
- *     Attach the #GstControlSource on the controller to a property.
- *     gst_controller_set_control_source (controller, "prop1", csource);
- *   </para></listitem>
- *   <listitem><para>
- *     Set the control values
  *     gst_interpolation_control_source_set (csource,0 * GST_SECOND, value1);
  *     gst_interpolation_control_source_set (csource,1 * GST_SECOND, value2);
  *   </para></listitem>
  *   <listitem><para>
+ *     Set the #GstControlSource in the controller.
+ *     gst_controller_set_control_source (controller, "prop1", csource);
+ *   </para></listitem>
+ *   <listitem><para>
  *     start your pipeline
  *   </para></listitem>
  * </orderedlist>
@@ -80,7 +77,6 @@
 #include "gstcontrollerprivate.h"
 #include "gstcontrolsource.h"
 #include "gstinterpolationcontrolsource.h"
-#include "gst/glib-compat-private.h"
 
 #define GST_CAT_DEFAULT controller_debug
 GST_DEBUG_CATEGORY_EXTERN (GST_CAT_DEFAULT);
@@ -101,21 +97,6 @@ struct _GstControllerPrivate
 };
 
 /* helper */
-#ifndef GST_REMOVE_DEPRECATED
-static void
-gst_controlled_property_add_interpolation_control_source (GstControlledProperty
-    * self)
-{
-  GstControlSource *csource =
-      GST_CONTROL_SOURCE (gst_interpolation_control_source_new ());
-
-  GST_INFO
-      ("Adding a GstInterpolationControlSource because of backward compatibility");
-  g_return_if_fail (!self->csource);
-  gst_control_source_bind (GST_CONTROL_SOURCE (csource), self->pspec);
-  self->csource = csource;
-}
-#endif
 
 /*
  * gst_controlled_property_new:
@@ -976,318 +957,3 @@ gst_controller_get_type (void)
   }
   return type;
 }
-
-/* FIXME: backward compatibility functions */
-
-/*
- * gst_controlled_property_set_interpolation_mode:
- * @self: the controlled property object to change
- * @mode: the new interpolation mode
- *
- * Sets the given Interpolation mode for the controlled property and activates
- * the respective interpolation hooks.
- *
- * Deprecated: Use #GstControlSource, for example #GstInterpolationControlSource
- * directly.
- *
- * Returns: %TRUE for success
- */
-#ifndef GST_REMOVE_DEPRECATED
-static gboolean
-gst_controlled_property_set_interpolation_mode (GstControlledProperty * self,
-    GstInterpolateMode mode)
-{
-  GstInterpolationControlSource *icsource;
-
-  /* FIXME: backward compat, add GstInterpolationControlSource */
-  if (!self->csource)
-    gst_controlled_property_add_interpolation_control_source (self);
-
-  g_return_val_if_fail (GST_IS_INTERPOLATION_CONTROL_SOURCE (self->csource),
-      FALSE);
-
-  icsource = GST_INTERPOLATION_CONTROL_SOURCE (self->csource);
-
-  return gst_interpolation_control_source_set_interpolation_mode (icsource,
-      mode);
-}
-#endif
-
-/**
- * gst_controller_set:
- * @self: the controller object which handles the properties
- * @property_name: the name of the property to set
- * @timestamp: the time the control-change is schedules for
- * @value: the control-value
- *
- * Set the value of given controller-handled property at a certain time.
- *
- * Deprecated: Use #GstControlSource, for example #GstInterpolationControlSource
- * directly.
- *
- * Returns: FALSE if the values couldn't be set (ex : properties not handled by controller), TRUE otherwise
- */
-#ifndef GST_REMOVE_DEPRECATED
-#ifdef GST_DISABLE_DEPRECATED
-gboolean
-gst_controller_set (GstController * self, const gchar * property_name,
-    GstClockTime timestamp, GValue * value);
-#endif
-gboolean
-gst_controller_set (GstController * self, const gchar * property_name,
-    GstClockTime timestamp, GValue * value)
-{
-  gboolean res = FALSE;
-  GstControlledProperty *prop;
-
-  g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
-  g_return_val_if_fail (property_name, FALSE);
-
-  g_mutex_lock (self->lock);
-  if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    /* FIXME: backward compat, add GstInterpolationControlSource */
-    if (!prop->csource)
-      gst_controlled_property_add_interpolation_control_source (prop);
-
-    if (!GST_IS_INTERPOLATION_CONTROL_SOURCE (prop->csource))
-      goto out;
-    res =
-        gst_interpolation_control_source_set (GST_INTERPOLATION_CONTROL_SOURCE
-        (prop->csource), timestamp, value);
-  }
-
-out:
-  g_mutex_unlock (self->lock);
-
-  return res;
-}
-#endif
-
-/**
- * gst_controller_set_from_list:
- * @self: the controller object which handles the properties
- * @property_name: the name of the property to set
- * @timedvalues: a list with #GstTimedValue items
- *
- * Sets multiple timed values at once.
- *
- * Deprecated: Use #GstControlSource, for example #GstInterpolationControlSource
- * directly.
- *
- * Returns: %FALSE if the values couldn't be set (ex : properties not handled by controller), %TRUE otherwise
- */
-#ifndef GST_REMOVE_DEPRECATED
-#ifdef GST_DISABLE_DEPRECATED
-gboolean
-gst_controller_set_from_list (GstController * self, const gchar * property_name,
-    GSList * timedvalues);
-#endif
-gboolean
-gst_controller_set_from_list (GstController * self, const gchar * property_name,
-    GSList * timedvalues)
-{
-  gboolean res = FALSE;
-  GstControlledProperty *prop;
-
-  g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
-  g_return_val_if_fail (property_name, FALSE);
-
-  g_mutex_lock (self->lock);
-  if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    /* FIXME: backward compat, add GstInterpolationControlSource */
-    if (!prop->csource)
-      gst_controlled_property_add_interpolation_control_source (prop);
-
-    if (!GST_IS_INTERPOLATION_CONTROL_SOURCE (prop->csource))
-      goto out;
-
-    res =
-        gst_interpolation_control_source_set_from_list
-        (GST_INTERPOLATION_CONTROL_SOURCE (prop->csource), timedvalues);
-  }
-
-out:
-  g_mutex_unlock (self->lock);
-
-  return res;
-}
-#endif
-
-/**
- * gst_controller_unset:
- * @self: the controller object which handles the properties
- * @property_name: the name of the property to unset
- * @timestamp: the time the control-change should be removed from
- *
- * Used to remove the value of given controller-handled property at a certain
- * time.
- *
- * Deprecated: Use #GstControlSource, for example #GstInterpolationControlSource
- * directly.
- *
- * Returns: %FALSE if the values couldn't be unset (ex : properties not handled by controller), %TRUE otherwise
- */
-#ifndef GST_REMOVE_DEPRECATED
-#ifdef GST_DISABLE_DEPRECATED
-gboolean
-gst_controller_unset (GstController * self, const gchar * property_name,
-    GstClockTime timestamp);
-#endif
-gboolean
-gst_controller_unset (GstController * self, const gchar * property_name,
-    GstClockTime timestamp)
-{
-  gboolean res = FALSE;
-  GstControlledProperty *prop;
-
-  g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
-  g_return_val_if_fail (property_name, FALSE);
-  g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (timestamp), FALSE);
-
-  g_mutex_lock (self->lock);
-  if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    if (!prop->csource || !GST_IS_INTERPOLATION_CONTROL_SOURCE (prop->csource))
-      goto out;
-
-    res =
-        gst_interpolation_control_source_unset (GST_INTERPOLATION_CONTROL_SOURCE
-        (prop->csource), timestamp);
-  }
-
-out:
-  g_mutex_unlock (self->lock);
-
-  return res;
-}
-#endif
-
-/**
- * gst_controller_unset_all:
- * @self: the controller object which handles the properties
- * @property_name: the name of the property to unset
- *
- * Used to remove all time-stamped values of given controller-handled property
- *
- * Deprecated: Use #GstControlSource, for example #GstInterpolationControlSource
- * directly.
- *
- * Returns: %FALSE if the values couldn't be unset (ex : properties not handled
- * by controller), %TRUE otherwise
- * Since: 0.10.5
- */
-#ifndef GST_REMOVE_DEPRECATED
-#ifdef GST_DISABLE_DEPRECATED
-gboolean gst_controller_unset_all (GstController * self,
-    const gchar * property_name);
-#endif
-gboolean
-gst_controller_unset_all (GstController * self, const gchar * property_name)
-{
-  GstControlledProperty *prop;
-
-  g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
-  g_return_val_if_fail (property_name, FALSE);
-
-  g_mutex_lock (self->lock);
-  if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    if (!prop->csource || !GST_IS_INTERPOLATION_CONTROL_SOURCE (prop->csource))
-      goto out;
-
-    gst_interpolation_control_source_unset_all (GST_INTERPOLATION_CONTROL_SOURCE
-        (prop->csource));
-  }
-
-out:
-  g_mutex_unlock (self->lock);
-
-  return TRUE;
-}
-#endif
-
-/**
- * gst_controller_get_all:
- * @self: the controller to get the list from
- * @property_name: the name of the property to get the list for
- *
- * Returns a read-only copy of the list of #GstTimedValue for the given property.
- * Free the list after done with it.
- *
- * <note><para>This doesn't modify the controlled GObject property!</para></note>
- *
- * Deprecated: Use #GstControlSource, for example #GstInterpolationControlSource
- * directly.
- *
- * Returns: a copy of the list, or %NULL if the property isn't handled by the controller
- */
-#ifndef GST_REMOVE_DEPRECATED
-#ifdef GST_DISABLE_DEPRECATED
-const GList *gst_controller_get_all (GstController * self,
-    const gchar * property_name);
-#endif
-const GList *
-gst_controller_get_all (GstController * self, const gchar * property_name)
-{
-  const GList *res = NULL;
-  GstControlledProperty *prop;
-
-  g_return_val_if_fail (GST_IS_CONTROLLER (self), NULL);
-  g_return_val_if_fail (property_name, NULL);
-
-  g_mutex_lock (self->lock);
-  if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    if (!prop->csource || !GST_IS_INTERPOLATION_CONTROL_SOURCE (prop->csource))
-      goto out;
-
-    res =
-        gst_interpolation_control_source_get_all
-        (GST_INTERPOLATION_CONTROL_SOURCE (prop->csource));
-  }
-
-out:
-  g_mutex_unlock (self->lock);
-
-  return res;
-}
-#endif
-
-/**
- * gst_controller_set_interpolation_mode:
- * @self: the controller object
- * @property_name: the name of the property for which to change the interpolation
- * @mode: interpolation mode
- *
- * Sets the given interpolation mode on the given property.
- *
- * <note><para>User interpolation is not yet available and quadratic interpolation
- * is deprecated and maps to cubic interpolation.</para></note>
- *
- * Deprecated: Use #GstControlSource, for example #GstInterpolationControlSource
- * directly.
- *
- * Returns: %TRUE if the property is handled by the controller, %FALSE otherwise
- */
-#ifndef GST_REMOVE_DEPRECATED
-#ifdef GST_DISABLE_DEPRECATED
-gboolean
-gst_controller_set_interpolation_mode (GstController * self,
-    const gchar * property_name, GstInterpolateMode mode);
-#endif
-gboolean
-gst_controller_set_interpolation_mode (GstController * self,
-    const gchar * property_name, GstInterpolateMode mode)
-{
-  gboolean res = FALSE;
-  GstControlledProperty *prop;
-
-  g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
-  g_return_val_if_fail (property_name, FALSE);
-
-  g_mutex_lock (self->lock);
-  if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    res = gst_controlled_property_set_interpolation_mode (prop, mode);
-  }
-  g_mutex_unlock (self->lock);
-
-  return res;
-}
-#endif
