@@ -105,10 +105,11 @@ static void gst_net_time_provider_set_property (GObject * object, guint prop_id,
 static void gst_net_time_provider_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-#define _do_init(type) \
+#define _do_init \
   GST_DEBUG_CATEGORY_INIT (ntp_debug, "nettime", 0, "Network time provider");
 
-GST_BOILERPLATE_FULL (GstNetTimeProvider, gst_net_time_provider, GstObject,
+#define gst_net_time_provider_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstNetTimeProvider, gst_net_time_provider,
     GST_TYPE_OBJECT, _do_init);
 
 #ifdef G_OS_WIN32
@@ -123,12 +124,6 @@ inet_aton (const char *c, struct in_addr *paddr)
 }
 #endif
 
-static void
-gst_net_time_provider_base_init (gpointer g_class)
-{
-  /* Do nothing here */
-}
-
 static void
 gst_net_time_provider_class_init (GstNetTimeProviderClass * klass)
 {
@@ -163,8 +158,7 @@ gst_net_time_provider_class_init (GstNetTimeProviderClass * klass)
 }
 
 static void
-gst_net_time_provider_init (GstNetTimeProvider * self,
-    GstNetTimeProviderClass * g_class)
+gst_net_time_provider_init (GstNetTimeProvider * self)
 {
 #ifdef G_OS_WIN32
   WSADATA w;
@@ -355,7 +349,7 @@ gst_net_time_provider_start (GstNetTimeProvider * self)
   socklen_t len;
   int port;
   gint ret;
-  GError *error = NULL;
+  GError *error;
 
   if ((ret = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
     goto no_socket;
@@ -403,15 +397,9 @@ gst_net_time_provider_start (GstNetTimeProvider * self)
   gst_poll_add_fd (self->priv->fdset, &self->priv->sock);
   gst_poll_fd_ctl_read (self->priv->fdset, &self->priv->sock, TRUE);
 
-#if !GLIB_CHECK_VERSION (2, 31, 0)
   self->thread = g_thread_create (gst_net_time_provider_thread, self, TRUE,
       &error);
-#else
-  self->thread = g_thread_try_new ("GstNetTimeProvider",
-      gst_net_time_provider_thread, self, &error);
-#endif
-
-  if (error != NULL)
+  if (!self->thread)
     goto no_thread;
 
   return TRUE;
