@@ -34,8 +34,9 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS_ANY);
 
 /* Data probe cb to drop everything but count buffers and events */
-static gboolean
-probe_cb (GstPad * pad, GstMiniObject * obj, gpointer user_data)
+static GstProbeReturn
+probe_cb (GstPad * pad, GstProbeType type, GstMiniObject * obj,
+    gpointer user_data)
 {
   gint count = 0;
   const gchar *count_type = NULL;
@@ -56,7 +57,7 @@ probe_cb (GstPad * pad, GstMiniObject * obj, gpointer user_data)
   g_object_set_data (G_OBJECT (pad), count_type, GINT_TO_POINTER (count));
 
   /* drop everything */
-  return FALSE;
+  return GST_PROBE_DROP;
 }
 
 /* Create and link output pad: selector:src%d ! output_pad */
@@ -74,7 +75,9 @@ setup_output_pad (GstElement * element, GstStaticPadTemplate * tmpl)
   fail_if (output_pad == NULL, "Could not create a output_pad");
 
   /* add probe */
-  probe_id = gst_pad_add_data_probe (output_pad, G_CALLBACK (probe_cb), NULL);
+  probe_id =
+      gst_pad_add_probe (output_pad, GST_PROBE_TYPE_DATA,
+      (GstPadProbeCallback) probe_cb, NULL, NULL);
   g_object_set_data (G_OBJECT (output_pad), "probe_id",
       GINT_TO_POINTER (probe_id));
 
@@ -110,7 +113,7 @@ cleanup_pad (GstPad * pad, GstElement * element)
   /* remove probe if necessary */
   probe_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (pad), "probe_id"));
   if (probe_id)
-    gst_pad_remove_data_probe (pad, probe_id);
+    gst_pad_remove_probe (pad, probe_id);
 
   /* unlink */
   selpad = gst_pad_get_peer (pad);
@@ -120,9 +123,6 @@ cleanup_pad (GstPad * pad, GstElement * element)
     gst_pad_unlink (pad, selpad);
   }
 
-  /* caps could have been set, make sure they get unset */
-  gst_pad_set_caps (pad, NULL);
-
   GST_DEBUG_OBJECT (pad, "clean up %" GST_PTR_FORMAT " and  %" GST_PTR_FORMAT,
       selpad, pad);
 
@@ -198,19 +198,6 @@ selector_set_active_pad (GstElement * elem, GstPad * selpad)
   g_free (padname);
 }
 
-static void
-push_newsegment_events (GList * input_pads)
-{
-  GList *l;
-
-  for (l = input_pads; l; l = l->next) {
-    GstPad *pad = l->data;
-
-    gst_pad_push_event (pad, gst_event_new_new_segment_full (FALSE, 1.0, 1.0,
-            GST_FORMAT_BYTES, 0, -1, 0));
-  }
-}
-
 /* Push buffers and switch for each selector pad */
 static void
 push_switched_buffers (GList * input_pads,
@@ -224,8 +211,6 @@ push_switched_buffers (GList * input_pads,
   /* setup dummy buffer */
   caps = gst_caps_from_string ("application/x-unknown");
   buf = gst_buffer_new_and_alloc (1);
-  gst_buffer_set_caps (buf, caps);
-  gst_caps_unref (caps);
 
   while (l != NULL) {
     /* set selector pad */
@@ -242,6 +227,7 @@ push_switched_buffers (GList * input_pads,
 
   /* cleanup buffer */
   gst_buffer_unref (buf);
+  gst_caps_unref (caps);
 }
 
 /* Create output-selector with given number of src pads and switch
@@ -267,7 +253,6 @@ run_output_selector_buffer_count (gint num_output_pads,
   fail_unless (gst_element_set_state (sel,
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
-  push_newsegment_events (input_pads);
   push_switched_buffers (input_pads, sel, output_pads, num_buffers_per_output);
   count_output_buffers (output_pads, num_buffers_per_output);
   fail_unless (gst_element_set_state (sel,
@@ -331,7 +316,9 @@ run_input_selector_buffer_count (gint num_input_pads,
     input_pads = g_list_append (input_pads, setup_input_pad (sel));
   }
   /* add probe */
-  probe_id = gst_pad_add_data_probe (output_pad, G_CALLBACK (probe_cb), NULL);
+  probe_id =
+      gst_pad_add_probe (output_pad, GST_PROBE_TYPE_DATA,
+      (GstPadProbeCallback) probe_cb, NULL, NULL);
   g_object_set_data (G_OBJECT (output_pad), "probe_id",
       GINT_TO_POINTER (probe_id));
 
@@ -339,14 +326,13 @@ run_input_selector_buffer_count (gint num_input_pads,
   fail_unless (gst_element_set_state (sel,
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
-  push_newsegment_events (input_pads);
   push_switched_buffers (input_pads, sel, input_pads, num_buffers_per_input);
   count_output_buffers (output_pads, (num_input_pads * num_buffers_per_input));
   fail_unless (gst_element_set_state (sel,
           GST_STATE_NULL) == GST_STATE_CHANGE_SUCCESS, "could not set to null");
 
   /* clean up */
-  gst_pad_remove_data_probe (output_pad, probe_id);
+  gst_pad_remove_probe (output_pad, probe_id);
   gst_pad_set_active (output_pad, FALSE);
   gst_check_teardown_sink_pad (sel);
   GST_DEBUG ("setting selector pad to NULL");
@@ -411,7 +397,7 @@ GST_START_TEST (test_output_selector_no_srcpad_negotiation);
      * setcaps should accept any caps when there are no srcpads */
     g_object_set (sel, "pad-negotiation-mode", i, NULL);
 
-    caps = gst_pad_get_caps (pad);
+    caps = gst_pad_get_caps (pad, NULL);
     fail_unless (gst_caps_is_any (caps));
 
     gst_caps_unref (caps);
@@ -488,7 +474,7 @@ GST_START_TEST (test_output_selector_getcaps_none);
 
     g_object_set (sel, "active-pad", pad, NULL);
 
-    caps = gst_pad_peer_get_caps (input_pad);
+    caps = gst_pad_peer_get_caps (input_pad, NULL);
 
     /* in 'none' mode, the getcaps returns the template, which is ANY */
     g_assert (gst_caps_is_any (caps));
@@ -528,7 +514,7 @@ GST_START_TEST (test_output_selector_getcaps_all);
 
     g_object_set (sel, "active-pad", pad, NULL);
 
-    caps = gst_pad_peer_get_caps (input_pad);
+    caps = gst_pad_peer_get_caps (input_pad, NULL);
 
     g_assert (gst_caps_is_equal (caps, expected));
     gst_caps_unref (caps);
@@ -569,10 +555,11 @@ GST_START_TEST (test_output_selector_getcaps_active);
 
     expected = gst_pad_template_get_caps (gst_pad_get_pad_template ((GstPad *)
             walker->data));
-    caps = gst_pad_peer_get_caps (input_pad);
+    caps = gst_pad_peer_get_caps (input_pad, NULL);
 
     g_assert (gst_caps_is_equal (caps, expected));
     gst_caps_unref (caps);
+    gst_caps_unref (expected);
     gst_object_unref (pad);
   }
 
