@@ -763,13 +763,16 @@ GST_END_TEST;
 static GMutex *blocked_lock;
 static GCond *blocked_cond;
 
-static void
-pad_blocked_cb (GstPad * pad, gboolean blocked, gpointer user_data)
+static GstProbeReturn
+pad_blocked_cb (GstPad * pad, GstProbeType type, gpointer type_data,
+    gpointer user_data)
 {
   g_mutex_lock (blocked_lock);
-  GST_DEBUG ("srcpad blocked: %d, sending signal", blocked);
+  GST_DEBUG ("srcpad blocked: %d, sending signal", type);
   g_cond_signal (blocked_cond);
   g_mutex_unlock (blocked_lock);
+
+  return GST_PROBE_OK;
 }
 
 GST_START_TEST (test_add_live2)
@@ -778,6 +781,7 @@ GST_START_TEST (test_add_live2)
   GstStateChangeReturn ret;
   GstState current, pending;
   GstPad *srcpad, *sinkpad;
+  gulong id;
 
   blocked_lock = g_mutex_new ();
   blocked_cond = g_cond_new ();
@@ -799,7 +803,8 @@ GST_START_TEST (test_add_live2)
   GST_DEBUG ("blocking srcpad");
   /* block source pad */
   srcpad = gst_element_get_static_pad (src, "src");
-  gst_pad_set_blocked_async (srcpad, TRUE, pad_blocked_cb, NULL);
+  id = gst_pad_add_probe (srcpad, GST_PROBE_TYPE_BLOCK, pad_blocked_cb, NULL,
+      NULL);
 
   /* set source to PAUSED without adding it to the pipeline */
   ret = gst_element_set_state (src, GST_STATE_PAUSED);
@@ -827,7 +832,7 @@ GST_START_TEST (test_add_live2)
   GST_DEBUG ("unblocking srcpad");
 
   /* and unblock */
-  gst_pad_set_blocked_async (srcpad, FALSE, pad_blocked_cb, NULL);
+  gst_pad_remove_probe (srcpad, id);
 
   GST_DEBUG ("getting state");
 
@@ -898,10 +903,14 @@ GST_START_TEST (test_bin_live)
   gst_object_unref (pipeline);
 }
 
-GST_END_TEST static void
+GST_END_TEST static gpointer
 send_eos (GstPad * sinkpad)
 {
-  gst_pad_send_event (sinkpad, gst_event_new_eos ());
+  gboolean ret;
+
+  ret = gst_pad_send_event (sinkpad, gst_event_new_eos ());
+
+  return GINT_TO_POINTER (ret);
 }
 
 /* push a buffer with a very long duration in a fakesink, then push an EOS
@@ -972,6 +981,9 @@ GST_START_TEST (test_fake_eos)
   ret = gst_element_set_state (pipeline, GST_STATE_NULL);
   fail_unless (ret == GST_STATE_CHANGE_SUCCESS, "no SUCCESS state return");
 
+  /* we can join now */
+  g_thread_join (thread);
+
   gst_object_unref (pipeline);
 }
 
@@ -995,15 +1007,13 @@ static GstBusSyncReply
 async_done_func (GstBus * bus, GstMessage * msg, GstElement * sink)
 {
   if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ASYNC_DONE) {
-    GstFormat format;
     gint64 position;
 
     GST_DEBUG ("we have ASYNC_DONE now");
     fail_unless (have_preroll == TRUE, "no preroll buffer received");
 
     /* get the position now */
-    format = GST_FORMAT_TIME;
-    gst_element_query_position (sink, &format, &position);
+    gst_element_query_position (sink, GST_FORMAT_TIME, &position);
 
     GST_DEBUG ("we have position %" GST_TIME_FORMAT, GST_TIME_ARGS (position));
 
@@ -1040,20 +1050,18 @@ send_buffer (GstPad * sinkpad)
 GST_START_TEST (test_async_done)
 {
   GstElement *sink;
-  GstBuffer *buffer;
   GstEvent *event;
   GstStateChangeReturn ret;
   GstPad *sinkpad;
   GstFlowReturn res;
   GstBus *bus;
   GThread *thread;
-  GstFormat format;
   gint64 position;
   gboolean qret;
+  GstSegment segment;
 
   sink = gst_element_factory_make ("fakesink", "sink");
   g_object_set (G_OBJECT (sink), "sync", TRUE, NULL);
-  g_object_set (G_OBJECT (sink), "preroll-queue-len", 2, NULL);
   g_object_set (G_OBJECT (sink), "signal-handoffs", TRUE, NULL);
 
   g_signal_connect (sink, "preroll-handoff", (GCallback) async_done_handoff,
@@ -1071,45 +1079,19 @@ GST_START_TEST (test_async_done)
 
   /* make newsegment, this sets the position to 10sec when the buffer prerolls */
   GST_DEBUG ("sending segment");
-  event =
-      gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME, 0, -1,
-      10 * GST_SECOND);
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+  segment.time = 10 * GST_SECOND;
+
+  event = gst_event_new_segment (&segment);
   res = gst_pad_send_event (sinkpad, event);
 
   /* We have not yet received any buffers so we are still in the READY state,
    * the position is therefore still not queryable. */
-  format = GST_FORMAT_TIME;
   position = -1;
-  qret = gst_element_query_position (sink, &format, &position);
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &position);
   fail_unless (qret == TRUE, "position wrong");
   fail_unless (position == 10 * GST_SECOND, "position is wrong");
 
-  /* Since we are paused and the preroll queue has a length of 2, this function
-   * will return immediately, the preroll handoff will be called and the stream
-   * position should now be 10 seconds. */
-  GST_DEBUG ("pushing first buffer");
-  buffer = gst_buffer_new_and_alloc (10);
-  GST_BUFFER_TIMESTAMP (buffer) = 1 * GST_SECOND;
-  GST_BUFFER_DURATION (buffer) = 100 * GST_SECOND;
-  res = gst_pad_chain (sinkpad, buffer);
-  fail_unless (res == GST_FLOW_OK, "no OK flow return");
-
-  /* scond buffer, will not block either but position should still be 10
-   * seconds */
-  GST_DEBUG ("pushing second buffer");
-  buffer = gst_buffer_new_and_alloc (10);
-  GST_BUFFER_TIMESTAMP (buffer) = 100 * GST_SECOND;
-  GST_BUFFER_DURATION (buffer) = 100 * GST_SECOND;
-  res = gst_pad_chain (sinkpad, buffer);
-  fail_unless (res == GST_FLOW_OK, "no OK flow return");
-
-  /* check if position is still 10 seconds */
-  format = GST_FORMAT_TIME;
-  gst_element_query_position (sink, &format, &position);
-  GST_DEBUG ("first buffer position %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (position));
-  fail_unless (position == 10 * GST_SECOND, "position is wrong");
-
   /* last buffer, blocks because preroll queue is filled. Start the push in a
    * new thread so that we can check the position */
   GST_DEBUG ("starting thread");
@@ -1123,8 +1105,7 @@ GST_START_TEST (test_async_done)
   /* check if position is still 10 seconds. This is racy because  the above
    * thread might not yet have started the push, because of the above sleep,
    * this is very unlikely, though. */
-  format = GST_FORMAT_TIME;
-  gst_element_query_position (sink, &format, &position);
+  gst_element_query_position (sink, GST_FORMAT_TIME, &position);
   GST_DEBUG ("second buffer position %" GST_TIME_FORMAT,
       GST_TIME_ARGS (position));
   fail_unless (position == 10 * GST_SECOND, "position is wrong");
@@ -1139,8 +1120,7 @@ GST_START_TEST (test_async_done)
   GST_DEBUG ("joining thread");
   g_thread_join (thread);
 
-  format = GST_FORMAT_TIME;
-  gst_element_query_position (sink, &format, &position);
+  gst_element_query_position (sink, GST_FORMAT_TIME, &position);
   GST_DEBUG ("last buffer position %" GST_TIME_FORMAT,
       GST_TIME_ARGS (position));
   fail_unless (position == 310 * GST_SECOND, "position is wrong");
@@ -1159,14 +1139,12 @@ static GstBusSyncReply
 async_done_eos_func (GstBus * bus, GstMessage * msg, GstElement * sink)
 {
   if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ASYNC_DONE) {
-    GstFormat format;
     gint64 position;
 
     GST_DEBUG ("we have ASYNC_DONE now");
 
     /* get the position now */
-    format = GST_FORMAT_TIME;
-    gst_element_query_position (sink, &format, &position);
+    gst_element_query_position (sink, GST_FORMAT_TIME, &position);
 
     GST_DEBUG ("we have position %" GST_TIME_FORMAT, GST_TIME_ARGS (position));
 
@@ -1189,13 +1167,13 @@ GST_START_TEST (test_async_done_eos)
   GstPad *sinkpad;
   gboolean res;
   GstBus *bus;
-  GstFormat format;
-  gint64 position;
   gboolean qret;
+  GstSegment segment;
+  gint64 position;
+  GThread *thread;
 
   sink = gst_element_factory_make ("fakesink", "sink");
   g_object_set (G_OBJECT (sink), "sync", TRUE, NULL);
-  g_object_set (G_OBJECT (sink), "preroll-queue-len", 1, NULL);
 
   sinkpad = gst_element_get_static_pad (sink, "sink");
 
@@ -1209,16 +1187,15 @@ GST_START_TEST (test_async_done_eos)
 
   /* make newsegment, this sets the position to 10sec when the buffer prerolls */
   GST_DEBUG ("sending segment");
-  event =
-      gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME, 0, -1,
-      10 * GST_SECOND);
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+  segment.time = 10 * GST_SECOND;
+  event = gst_event_new_segment (&segment);
   res = gst_pad_send_event (sinkpad, event);
 
   /* We have not yet received any buffers so we are still in the READY state,
    * the position is therefore still not queryable. */
-  format = GST_FORMAT_TIME;
   position = -1;
-  qret = gst_element_query_position (sink, &format, &position);
+  qret = gst_element_query_position (sink, GST_FORMAT_TIME, &position);
   fail_unless (qret == TRUE, "position wrong");
   fail_unless (position == 10 * GST_SECOND, "position is wrong");
 
@@ -1226,19 +1203,22 @@ GST_START_TEST (test_async_done_eos)
    * will return immediately. The EOS will complete the preroll and the
    * position should now be 10 seconds. */
   GST_DEBUG ("pushing EOS");
-  event = gst_event_new_eos ();
-  res = gst_pad_send_event (sinkpad, event);
-  fail_unless (res == TRUE, "no TRUE return");
+  GST_DEBUG ("starting thread");
+  thread = g_thread_create ((GThreadFunc) send_eos, sinkpad, TRUE, NULL);
+  fail_if (thread == NULL, "no thread");
+
+  /* wait for preroll */
+  gst_element_get_state (sink, NULL, NULL, -1);
 
   /* check if position is still 10 seconds */
-  format = GST_FORMAT_TIME;
-  gst_element_query_position (sink, &format, &position);
+  gst_element_query_position (sink, GST_FORMAT_TIME, &position);
   GST_DEBUG ("EOS position %" GST_TIME_FORMAT, GST_TIME_ARGS (position));
   fail_unless (position == 10 * GST_SECOND, "position is wrong");
 
-  gst_object_unref (sinkpad);
-
   gst_element_set_state (sink, GST_STATE_NULL);
+  g_thread_join (thread);
+
+  gst_object_unref (sinkpad);
   gst_object_unref (sink);
   gst_object_unref (bus);
 }
