@@ -53,34 +53,13 @@ GST_START_TEST (test_from_string)
 
 GST_END_TEST;
 
-GST_START_TEST (test_buffer)
-{
-  GstCaps *c1;
-  GstBuffer *buffer;
-
-  buffer = gst_buffer_new_and_alloc (1000);
-  c1 = gst_caps_new_simple ("audio/x-raw-int",
-      "buffer", GST_TYPE_BUFFER, buffer, NULL);
-
-  GST_DEBUG ("caps: %" GST_PTR_FORMAT, c1);
-  gst_buffer_unref (buffer);
-
-  buffer = gst_buffer_new_and_alloc (1000);
-  gst_buffer_set_caps (buffer, c1);     /* doesn't give away our c1 ref */
-
-  gst_caps_unref (c1);
-  gst_buffer_unref (buffer);    /* Should now drop both references */
-}
-
-GST_END_TEST;
-
 GST_START_TEST (test_double_append)
 {
   GstStructure *s1;
   GstCaps *c1;
 
   c1 = gst_caps_new_any ();
-  s1 = gst_structure_from_string ("audio/x-raw-int,rate=44100", NULL);
+  s1 = gst_structure_from_string ("audio/x-raw,rate=44100", NULL);
   gst_caps_append_structure (c1, s1);
   ASSERT_CRITICAL (gst_caps_append_structure (c1, s1));
 
@@ -96,7 +75,7 @@ GST_START_TEST (test_mutability)
   gint ret;
 
   c1 = gst_caps_new_any ();
-  s1 = gst_structure_from_string ("audio/x-raw-int,rate=44100", NULL);
+  s1 = gst_structure_from_string ("audio/x-raw,rate=44100", NULL);
   gst_structure_set (s1, "rate", G_TYPE_INT, 48000, NULL);
   gst_caps_append_structure (c1, s1);
   gst_structure_set (s1, "rate", G_TYPE_INT, 22500, NULL);
@@ -121,7 +100,7 @@ GST_END_TEST;
 
 GST_START_TEST (test_static_caps)
 {
-  static GstStaticCaps scaps = GST_STATIC_CAPS ("audio/x-raw-int,rate=44100");
+  static GstStaticCaps scaps = GST_STATIC_CAPS ("audio/x-raw,rate=44100");
   GstCaps *caps1;
   GstCaps *caps2;
 
@@ -146,57 +125,71 @@ GST_START_TEST (test_static_caps)
 GST_END_TEST;
 
 static const gchar non_simple_caps_string[] =
-    "video/x-raw-yuv, format=(fourcc)I420, framerate=(fraction)[ 1/100, 100 ], "
-    "width=(int)[ 16, 4096 ], height=(int)[ 16, 4096 ]; video/x-raw-yuv, "
-    "format=(fourcc)YUY2, framerate=(fraction)[ 1/100, 100 ], width=(int)[ 16, 4096 ], "
-    "height=(int)[ 16, 4096 ]; video/x-raw-rgb, bpp=(int)8, depth=(int)8, "
-    "endianness=(int)1234, framerate=(fraction)[ 1/100, 100 ], width=(int)[ 16, 4096 ], "
-    "height=(int)[ 16, 4096 ]; video/x-raw-yuv, "
-    "format=(fourcc){ I420, YUY2, YV12 }, width=(int)[ 16, 4096 ], "
+    "video/x-raw, format=(string)I420, framerate=(fraction)[ 1/100, 100 ], "
+    "width=(int)[ 16, 4096 ], height=(int)[ 16, 4096 ]; video/x-raw, "
+    "format=(string)YUY2, framerate=(fraction)[ 1/100, 100 ], width=(int)[ 16, 4096 ], "
+    "height=(int)[ 16, 4096 ]; video/x-raw, format=(string)RGB8_PALETTED, "
+    "framerate=(fraction)[ 1/100, 100 ], width=(int)[ 16, 4096 ], "
+    "height=(int)[ 16, 4096 ]; video/x-raw, "
+    "format=(string){ I420, YUY2, YV12 }, width=(int)[ 16, 4096 ], "
     "height=(int)[ 16, 4096 ], framerate=(fraction)[ 1/100, 100 ]";
 
 static gboolean
-check_fourcc_list (const GValue * format_value)
+check_string_list (const GValue * format_value)
 {
-  const GValue *fourcc_value;
+  const GValue *string_value;
+  gboolean got_rgb8 = FALSE;
   gboolean got_yv12 = FALSE;
   gboolean got_i420 = FALSE;
   gboolean got_yuy2 = FALSE;
-  guint32 fourcc;
+  const gchar *string;
 
-  fourcc_value = gst_value_list_get_value (format_value, 0);
-  fail_unless (fourcc_value != NULL);
-  fail_unless (GST_VALUE_HOLDS_FOURCC (fourcc_value));
-  fourcc = gst_value_get_fourcc (fourcc_value);
-  fail_unless (fourcc != 0);
-  got_i420 = got_i420 || (fourcc == GST_STR_FOURCC ("I420"));
-  got_yuy2 = got_yuy2 || (fourcc == GST_STR_FOURCC ("YUY2"));
-  got_yv12 = got_yv12 || (fourcc == GST_STR_FOURCC ("YV12"));
+  string_value = gst_value_list_get_value (format_value, 0);
+  fail_unless (string_value != NULL);
+  fail_unless (G_VALUE_HOLDS_STRING (string_value));
+  string = g_value_get_string (string_value);
+  fail_unless (string != NULL);
+  got_rgb8 = got_rgb8 || (g_str_equal (string, "RGB8_PALETTED"));
+  got_i420 = got_i420 || (g_str_equal (string, "I420"));
+  got_yuy2 = got_yuy2 || (g_str_equal (string, "YUY2"));
+  got_yv12 = got_yv12 || (g_str_equal (string, "YV12"));
 
-  fourcc_value = gst_value_list_get_value (format_value, 1);
-  fail_unless (fourcc_value != NULL);
-  fail_unless (GST_VALUE_HOLDS_FOURCC (fourcc_value));
-  fourcc = gst_value_get_fourcc (fourcc_value);
-  fail_unless (fourcc != 0);
-  got_i420 = got_i420 || (fourcc == GST_STR_FOURCC ("I420"));
-  got_yuy2 = got_yuy2 || (fourcc == GST_STR_FOURCC ("YUY2"));
-  got_yv12 = got_yv12 || (fourcc == GST_STR_FOURCC ("YV12"));
+  string_value = gst_value_list_get_value (format_value, 1);
+  fail_unless (string_value != NULL);
+  fail_unless (G_VALUE_HOLDS_STRING (string_value));
+  string = g_value_get_string (string_value);
+  fail_unless (string != NULL);
+  got_rgb8 = got_rgb8 || (g_str_equal (string, "RGB8_PALETTED"));
+  got_i420 = got_i420 || (g_str_equal (string, "I420"));
+  got_yuy2 = got_yuy2 || (g_str_equal (string, "YUY2"));
+  got_yv12 = got_yv12 || (g_str_equal (string, "YV12"));
 
-  fourcc_value = gst_value_list_get_value (format_value, 2);
-  fail_unless (fourcc_value != NULL);
-  fail_unless (GST_VALUE_HOLDS_FOURCC (fourcc_value));
-  fourcc = gst_value_get_fourcc (fourcc_value);
-  fail_unless (fourcc != 0);
-  got_i420 = got_i420 || (fourcc == GST_STR_FOURCC ("I420"));
-  got_yuy2 = got_yuy2 || (fourcc == GST_STR_FOURCC ("YUY2"));
-  got_yv12 = got_yv12 || (fourcc == GST_STR_FOURCC ("YV12"));
+  string_value = gst_value_list_get_value (format_value, 2);
+  fail_unless (string_value != NULL);
+  fail_unless (G_VALUE_HOLDS_STRING (string_value));
+  string = g_value_get_string (string_value);
+  fail_unless (string != NULL);
+  got_rgb8 = got_rgb8 || (g_str_equal (string, "RGB8_PALETTED"));
+  got_i420 = got_i420 || (g_str_equal (string, "I420"));
+  got_yuy2 = got_yuy2 || (g_str_equal (string, "YUY2"));
+  got_yv12 = got_yv12 || (g_str_equal (string, "YV12"));
 
-  return (got_i420 && got_yuy2 && got_yv12);
+  string_value = gst_value_list_get_value (format_value, 3);
+  fail_unless (string_value != NULL);
+  fail_unless (G_VALUE_HOLDS_STRING (string_value));
+  string = g_value_get_string (string_value);
+  fail_unless (string != NULL);
+  got_rgb8 = got_rgb8 || (g_str_equal (string, "RGB8_PALETTED"));
+  got_i420 = got_i420 || (g_str_equal (string, "I420"));
+  got_yuy2 = got_yuy2 || (g_str_equal (string, "YUY2"));
+  got_yv12 = got_yv12 || (g_str_equal (string, "YV12"));
+
+  return (got_rgb8 && got_i420 && got_yuy2 && got_yv12);
 }
 
 GST_START_TEST (test_simplify)
 {
-  GstStructure *s1, *s2;
+  GstStructure *s1;
   gboolean did_simplify;
   GstCaps *caps;
 
@@ -210,46 +203,31 @@ GST_START_TEST (test_simplify)
 
   /* check simplified caps, should be:
    *
-   * video/x-raw-rgb, bpp=(int)8, depth=(int)8, endianness=(int)1234,
-   *     framerate=(fraction)[ 1/100, 100 ], width=(int)[ 16, 4096 ],
-   *     height=(int)[ 16, 4096 ];
-   * video/x-raw-yuv, format=(fourcc){ YV12, YUY2, I420 },
+   * video/x-raw, format=(string){ RGB8_PALETTED, YV12, YUY2, I420 },
    *     width=(int)[ 16, 4096 ], height=(int)[ 16, 4096 ],
    *     framerate=(fraction)[ 1/100, 100 ]
    */
-  fail_unless (gst_caps_get_size (caps) == 2);
+  GST_DEBUG ("simplyfied %" GST_PTR_FORMAT, caps);
+  fail_unless (gst_caps_get_size (caps) == 1);
   s1 = gst_caps_get_structure (caps, 0);
-  s2 = gst_caps_get_structure (caps, 1);
   fail_unless (s1 != NULL);
-  fail_unless (s2 != NULL);
 
-  if (!gst_structure_has_name (s1, "video/x-raw-rgb")) {
-    GstStructure *tmp;
-
-    tmp = s1;
-    s1 = s2;
-    s2 = tmp;
-  }
-
-  fail_unless (gst_structure_has_name (s1, "video/x-raw-rgb"));
+  fail_unless (gst_structure_has_name (s1, "video/x-raw"));
   {
     const GValue *framerate_value;
+    const GValue *format_value;
     const GValue *width_value;
     const GValue *height_value;
     const GValue *val_fps;
     GValue test_fps = { 0, };
-    gint bpp, depth, endianness;
     gint min_width, max_width;
     gint min_height, max_height;
 
-    fail_unless (gst_structure_get_int (s1, "bpp", &bpp));
-    fail_unless (bpp == 8);
-
-    fail_unless (gst_structure_get_int (s1, "depth", &depth));
-    fail_unless (depth == 8);
-
-    fail_unless (gst_structure_get_int (s1, "endianness", &endianness));
-    fail_unless (endianness == G_LITTLE_ENDIAN);
+    format_value = gst_structure_get_value (s1, "format");
+    fail_unless (format_value != NULL);
+    fail_unless (GST_VALUE_HOLDS_LIST (format_value));
+    fail_unless (gst_value_list_get_size (format_value) == 4);
+    fail_unless (check_string_list (format_value) == TRUE);
 
     g_value_init (&test_fps, GST_TYPE_FRACTION);
     framerate_value = gst_structure_get_value (s1, "framerate");
@@ -281,53 +259,6 @@ GST_START_TEST (test_simplify)
     fail_unless (min_height == 16 && max_height == 4096);
   }
 
-  fail_unless (gst_structure_has_name (s2, "video/x-raw-yuv"));
-  {
-    const GValue *framerate_value;
-    const GValue *format_value;
-    const GValue *width_value;
-    const GValue *height_value;
-    const GValue *val_fps;
-    GValue test_fps = { 0, };
-    gint min_width, max_width;
-    gint min_height, max_height;
-
-    format_value = gst_structure_get_value (s2, "format");
-    fail_unless (format_value != NULL);
-    fail_unless (GST_VALUE_HOLDS_LIST (format_value));
-    fail_unless (gst_value_list_get_size (format_value) == 3);
-    fail_unless (check_fourcc_list (format_value) == TRUE);
-
-    g_value_init (&test_fps, GST_TYPE_FRACTION);
-    framerate_value = gst_structure_get_value (s2, "framerate");
-    fail_unless (framerate_value != NULL);
-    fail_unless (GST_VALUE_HOLDS_FRACTION_RANGE (framerate_value));
-
-    val_fps = gst_value_get_fraction_range_min (framerate_value);
-    gst_value_set_fraction (&test_fps, 1, 100);
-    fail_unless (gst_value_compare (&test_fps, val_fps) == GST_VALUE_EQUAL);
-
-    val_fps = gst_value_get_fraction_range_max (framerate_value);
-    gst_value_set_fraction (&test_fps, 100, 1);
-    fail_unless (gst_value_compare (&test_fps, val_fps) == GST_VALUE_EQUAL);
-
-    g_value_unset (&test_fps);
-
-    width_value = gst_structure_get_value (s2, "width");
-    fail_unless (width_value != NULL);
-    fail_unless (GST_VALUE_HOLDS_INT_RANGE (width_value));
-    min_width = gst_value_get_int_range_min (width_value);
-    max_width = gst_value_get_int_range_max (width_value);
-    fail_unless (min_width == 16 && max_width == 4096);
-
-    height_value = gst_structure_get_value (s2, "height");
-    fail_unless (height_value != NULL);
-    fail_unless (GST_VALUE_HOLDS_INT_RANGE (height_value));
-    min_height = gst_value_get_int_range_min (height_value);
-    max_height = gst_value_get_int_range_max (height_value);
-    fail_unless (min_height == 16 && max_height == 4096);
-  }
-
   gst_caps_unref (caps);
 }
 
@@ -352,24 +283,23 @@ GST_START_TEST (test_subset)
 {
   GstCaps *c1, *c2;
 
-  c1 = gst_caps_from_string ("video/x-raw-yuv; video/x-raw-rgb");
-  c2 = gst_caps_from_string ("video/x-raw-yuv, format=(fourcc)YUY2");
+  c1 = gst_caps_from_string ("video/x-raw; video/x-raw");
+  c2 = gst_caps_from_string ("video/x-raw, format=(string)YUY2");
   fail_unless (gst_caps_is_subset (c2, c1));
   fail_if (gst_caps_is_subset (c1, c2));
   gst_caps_unref (c1);
   gst_caps_unref (c2);
 
   c1 = gst_caps_from_string
-      ("audio/x-raw-int, channels=(int)[ 1, 2 ], rate=(int)44100");
-  c2 = gst_caps_from_string
-      ("audio/x-raw-int, channels=(int)1, rate=(int)44100");
+      ("audio/x-raw, channels=(int)[ 1, 2 ], rate=(int)44100");
+  c2 = gst_caps_from_string ("audio/x-raw, channels=(int)1, rate=(int)44100");
   fail_unless (gst_caps_is_subset (c2, c1));
   fail_if (gst_caps_is_subset (c1, c2));
   gst_caps_unref (c1);
   gst_caps_unref (c2);
 
-  c1 = gst_caps_from_string ("audio/x-raw-int, channels=(int) {1}");
-  c2 = gst_caps_from_string ("audio/x-raw-int, channels=(int)1");
+  c1 = gst_caps_from_string ("audio/x-raw, channels=(int) {1}");
+  c2 = gst_caps_from_string ("audio/x-raw, channels=(int)1");
   fail_unless (gst_caps_is_subset (c2, c1));
   fail_unless (gst_caps_is_subset (c1, c2));
   fail_unless (gst_caps_is_equal (c1, c2));
@@ -377,9 +307,9 @@ GST_START_TEST (test_subset)
   gst_caps_unref (c2);
 
   c1 = gst_caps_from_string
-      ("audio/x-raw-int, rate=(int)44100, channels=(int)3, endianness=(int)1234, width=(int)16, depth=(int)16, signed=(boolean)false");
+      ("audio/x-raw, rate=(int)44100, channels=(int)3, format=(string)U16_LE");
   c2 = gst_caps_from_string
-      ("audio/x-raw-int, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 2147483647 ], endianness=(int){ 1234, 4321 }, width=(int)16, depth=(int)[ 1, 16 ], signed=(boolean){ true, false }");
+      ("audio/x-raw, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 2147483647 ], format=(string){ S16_LE, U16_LE }");
   fail_unless (gst_caps_is_subset (c1, c2));
   fail_if (gst_caps_is_subset (c2, c1));
   gst_caps_unref (c1);
@@ -393,7 +323,7 @@ GST_START_TEST (test_merge_fundamental)
   GstCaps *c1, *c2;
 
   /* ANY + specific = ANY */
-  c1 = gst_caps_from_string ("audio/x-raw-int,rate=44100");
+  c1 = gst_caps_from_string ("audio/x-raw,rate=44100");
   c2 = gst_caps_new_any ();
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
@@ -402,7 +332,7 @@ GST_START_TEST (test_merge_fundamental)
   gst_caps_unref (c2);
 
   /* specific + ANY = ANY */
-  c2 = gst_caps_from_string ("audio/x-raw-int,rate=44100");
+  c2 = gst_caps_from_string ("audio/x-raw,rate=44100");
   c1 = gst_caps_new_any ();
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
@@ -411,7 +341,7 @@ GST_START_TEST (test_merge_fundamental)
   gst_caps_unref (c2);
 
   /* EMPTY + specific = specific */
-  c1 = gst_caps_from_string ("audio/x-raw-int,rate=44100");
+  c1 = gst_caps_from_string ("audio/x-raw,rate=44100");
   c2 = gst_caps_new_empty ();
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
@@ -420,7 +350,7 @@ GST_START_TEST (test_merge_fundamental)
   gst_caps_unref (c2);
 
   /* specific + EMPTY = specific */
-  c2 = gst_caps_from_string ("audio/x-raw-int,rate=44100");
+  c2 = gst_caps_from_string ("audio/x-raw,rate=44100");
   c1 = gst_caps_new_empty ();
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
@@ -436,19 +366,19 @@ GST_START_TEST (test_merge_same)
   GstCaps *c1, *c2, *test;
 
   /* this is the same */
-  c1 = gst_caps_from_string ("audio/x-raw-int,rate=44100,channels=1");
-  c2 = gst_caps_from_string ("audio/x-raw-int,rate=44100,channels=1");
+  c1 = gst_caps_from_string ("audio/x-raw,rate=44100,channels=1");
+  c2 = gst_caps_from_string ("audio/x-raw,rate=44100,channels=1");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 1, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,rate=44100,channels=1");
+  test = gst_caps_from_string ("audio/x-raw,rate=44100,channels=1");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (test);
   gst_caps_unref (c2);
 
   /* and so is this */
-  c1 = gst_caps_from_string ("audio/x-raw-int,rate=44100,channels=1");
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=1,rate=44100");
+  c1 = gst_caps_from_string ("audio/x-raw,rate=44100,channels=1");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=1,rate=44100");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 1, NULL);
@@ -490,123 +420,121 @@ GST_START_TEST (test_merge_subset)
   GstCaps *c1, *c2, *test;
 
   /* the 2nd is already covered */
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=[1,2]");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=1");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=[1,2]");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=1");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 1, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels=[1,2]");
+  test = gst_caps_from_string ("audio/x-raw,channels=[1,2]");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
   /* here it is not */
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=1,rate=44100");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=[1,2],rate=44100");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=1,rate=44100");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=[1,2],rate=44100");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 2, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels=[1,2],rate=44100");
+  test = gst_caps_from_string ("audio/x-raw,channels=[1,2],rate=44100");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
   /* second one was already contained in the first one */
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=[1,3]");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=[1,2]");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=[1,3]");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=[1,2]");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 1, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels=[1,3]");
+  test = gst_caps_from_string ("audio/x-raw,channels=[1,3]");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
   /* second one was already contained in the first one */
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=[1,4]");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=[1,2]");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=[1,4]");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=[1,2]");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 1, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels=[1,4]");
+  test = gst_caps_from_string ("audio/x-raw,channels=[1,4]");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
   /* second one was already contained in the first one */
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=[1,4]");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=[2,4]");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=[1,4]");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=[2,4]");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 1, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels=[1,4]");
+  test = gst_caps_from_string ("audio/x-raw,channels=[1,4]");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
   /* second one was already contained in the first one */
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=[1,4]");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=[2,3]");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=[1,4]");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=[2,3]");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 1, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels=[1,4]");
+  test = gst_caps_from_string ("audio/x-raw,channels=[1,4]");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
   /* these caps cannot be merged */
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=[2,3]");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=[1,4]");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=[2,3]");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=[1,4]");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 2, NULL);
   test =
       gst_caps_from_string
-      ("audio/x-raw-int,channels=[2,3];audio/x-raw-int,channels=[1,4]");
+      ("audio/x-raw,channels=[2,3];audio/x-raw,channels=[1,4]");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
   /* these caps cannot be merged */
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=[1,2]");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=[1,3]");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=[1,2]");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=[1,3]");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 2, NULL);
   test =
       gst_caps_from_string
-      ("audio/x-raw-int,channels=[1,2];audio/x-raw-int,channels=[1,3]");
+      ("audio/x-raw,channels=[1,2];audio/x-raw,channels=[1,3]");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels={1,2}");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels={1,2,3,4}");
+  c2 = gst_caps_from_string ("audio/x-raw,channels={1,2}");
+  c1 = gst_caps_from_string ("audio/x-raw,channels={1,2,3,4}");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 2, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels={1,2};"
-      "audio/x-raw-int,channels={1,2,3,4}");
+  test = gst_caps_from_string ("audio/x-raw,channels={1,2};"
+      "audio/x-raw,channels={1,2,3,4}");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels={1,2}");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels={1,3}");
+  c2 = gst_caps_from_string ("audio/x-raw,channels={1,2}");
+  c1 = gst_caps_from_string ("audio/x-raw,channels={1,3}");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 2, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels={1,2};"
-      "audio/x-raw-int,channels={1,3}");
+  test = gst_caps_from_string ("audio/x-raw,channels={1,2};"
+      "audio/x-raw,channels={1,3}");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
-  c2 = gst_caps_from_string
-      ("video/x-raw-yuv, framerate=(fraction){ 15/2, 5/1 }");
-  c1 = gst_caps_from_string
-      ("video/x-raw-yuv, framerate=(fraction){ 15/1, 5/1 }");
+  c2 = gst_caps_from_string ("video/x-raw, framerate=(fraction){ 15/2, 5/1 }");
+  c1 = gst_caps_from_string ("video/x-raw, framerate=(fraction){ 15/1, 5/1 }");
   test = gst_caps_copy (c1);
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
@@ -614,22 +542,22 @@ GST_START_TEST (test_merge_subset)
   gst_caps_unref (test);
   gst_caps_unref (c2);
 
-  c2 = gst_caps_from_string ("audio/x-raw-int");
-  c1 = gst_caps_from_string ("audio/x-raw-int,channels=1");
+  c2 = gst_caps_from_string ("audio/x-raw");
+  c1 = gst_caps_from_string ("audio/x-raw,channels=1");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 1, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int");
+  test = gst_caps_from_string ("audio/x-raw");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
 
-  c2 = gst_caps_from_string ("audio/x-raw-int,channels=1");
-  c1 = gst_caps_from_string ("audio/x-raw-int");
+  c2 = gst_caps_from_string ("audio/x-raw,channels=1");
+  c1 = gst_caps_from_string ("audio/x-raw");
   gst_caps_merge (c2, c1);
   GST_DEBUG ("merged: (%d) %" GST_PTR_FORMAT, gst_caps_get_size (c2), c2);
   fail_unless (gst_caps_get_size (c2) == 2, NULL);
-  test = gst_caps_from_string ("audio/x-raw-int,channels=1; audio/x-raw-int");
+  test = gst_caps_from_string ("audio/x-raw,channels=1; audio/x-raw");
   fail_unless (gst_caps_is_equal (c2, test));
   gst_caps_unref (c2);
   gst_caps_unref (test);
@@ -644,14 +572,14 @@ GST_START_TEST (test_intersect)
 
   /* field not specified = any value possible, so the intersection
    * should keep fields which are only part of one set of caps */
-  c2 = gst_caps_from_string ("video/x-raw-yuv,format=(fourcc)I420,width=20");
-  c1 = gst_caps_from_string ("video/x-raw-yuv,format=(fourcc)I420");
+  c2 = gst_caps_from_string ("video/x-raw,format=(string)I420,width=20");
+  c1 = gst_caps_from_string ("video/x-raw,format=(string)I420");
 
   ci1 = gst_caps_intersect (c2, c1);
   GST_DEBUG ("intersected: %" GST_PTR_FORMAT, ci1);
   fail_unless (gst_caps_get_size (ci1) == 1, NULL);
   s = gst_caps_get_structure (ci1, 0);
-  fail_unless (gst_structure_has_name (s, "video/x-raw-yuv"));
+  fail_unless (gst_structure_has_name (s, "video/x-raw"));
   fail_unless (gst_structure_get_value (s, "format") != NULL);
   fail_unless (gst_structure_get_value (s, "width") != NULL);
 
@@ -660,7 +588,7 @@ GST_START_TEST (test_intersect)
   GST_DEBUG ("intersected: %" GST_PTR_FORMAT, ci2);
   fail_unless (gst_caps_get_size (ci2) == 1, NULL);
   s = gst_caps_get_structure (ci2, 0);
-  fail_unless (gst_structure_has_name (s, "video/x-raw-yuv"));
+  fail_unless (gst_structure_has_name (s, "video/x-raw"));
   fail_unless (gst_structure_get_value (s, "format") != NULL);
   fail_unless (gst_structure_get_value (s, "width") != NULL);
 
@@ -674,8 +602,8 @@ GST_START_TEST (test_intersect)
 
   /* ========== */
 
-  c2 = gst_caps_from_string ("video/x-raw-yuv,format=(fourcc)I420,width=20");
-  c1 = gst_caps_from_string ("video/x-raw-yuv,format=(fourcc)I420,width=30");
+  c2 = gst_caps_from_string ("video/x-raw,format=(string)I420,width=20");
+  c1 = gst_caps_from_string ("video/x-raw,format=(string)I420,width=30");
 
   ci1 = gst_caps_intersect (c2, c1);
   GST_DEBUG ("intersected: %" GST_PTR_FORMAT, ci1);
@@ -696,8 +624,8 @@ GST_START_TEST (test_intersect)
 
   /* ========== */
 
-  c2 = gst_caps_from_string ("video/x-raw-yuv,format=(fourcc)I420,width=20");
-  c1 = gst_caps_from_string ("video/x-raw-rgb,format=(fourcc)I420,width=20");
+  c2 = gst_caps_from_string ("video/x-raw,format=(string)I420,width=20");
+  c1 = gst_caps_from_string ("video/x-raw2,format=(string)I420,width=20");
 
   ci1 = gst_caps_intersect (c2, c1);
   GST_DEBUG ("intersected: %" GST_PTR_FORMAT, ci1);
@@ -718,14 +646,14 @@ GST_START_TEST (test_intersect)
 
   /* ========== */
 
-  c2 = gst_caps_from_string ("video/x-raw-yuv,format=(fourcc)I420,width=20");
-  c1 = gst_caps_from_string ("video/x-raw-yuv,format=(fourcc)I420,height=30");
+  c2 = gst_caps_from_string ("video/x-raw,format=(string)I420,width=20");
+  c1 = gst_caps_from_string ("video/x-raw,format=(string)I420,height=30");
 
   ci1 = gst_caps_intersect (c2, c1);
   GST_DEBUG ("intersected: %" GST_PTR_FORMAT, ci1);
   fail_unless (gst_caps_get_size (ci1) == 1, NULL);
   s = gst_caps_get_structure (ci1, 0);
-  fail_unless (gst_structure_has_name (s, "video/x-raw-yuv"));
+  fail_unless (gst_structure_has_name (s, "video/x-raw"));
   fail_unless (gst_structure_get_value (s, "format") != NULL);
   fail_unless (gst_structure_get_value (s, "width") != NULL);
   fail_unless (gst_structure_get_value (s, "height") != NULL);
@@ -735,7 +663,7 @@ GST_START_TEST (test_intersect)
   GST_DEBUG ("intersected: %" GST_PTR_FORMAT, ci2);
   fail_unless (gst_caps_get_size (ci2) == 1, NULL);
   s = gst_caps_get_structure (ci2, 0);
-  fail_unless (gst_structure_has_name (s, "video/x-raw-yuv"));
+  fail_unless (gst_structure_has_name (s, "video/x-raw"));
   fail_unless (gst_structure_get_value (s, "format") != NULL);
   fail_unless (gst_structure_get_value (s, "height") != NULL);
   fail_unless (gst_structure_get_value (s, "width") != NULL);
@@ -756,14 +684,14 @@ GST_START_TEST (test_intersect2)
   GstCaps *caps1, *caps2, *icaps;
 
   /* tests array subtraction */
-  caps1 = gst_caps_from_string ("audio/x-raw-float, "
+  caps1 = gst_caps_from_string ("audio/x-raw, "
       "channel-positions=(int)<                      "
       "{ 1, 2, 3, 4, 5, 6 },                         "
       "{ 1, 2, 3, 4, 5, 6 },                         "
       "{ 1, 2, 3, 4, 5, 6 },                         "
       "{ 1, 2, 3, 4, 5, 6 },                         "
       "{ 1, 2, 3, 4, 5, 6 },                         " "{ 1, 2, 3, 4, 5, 6 }>");
-  caps2 = gst_caps_from_string ("audio/x-raw-float, "
+  caps2 = gst_caps_from_string ("audio/x-raw, "
       "channel-positions=(int)< 1, 2, 3, 4, 5, 6 >");
   icaps = gst_caps_intersect (caps1, caps2);
   GST_LOG ("intersected caps: %" GST_PTR_FORMAT, icaps);
@@ -876,9 +804,9 @@ GST_START_TEST (test_intersect_duplication)
   GstCaps *c1, *c2, *test;
 
   c1 = gst_caps_from_string
-      ("audio/x-raw-int, endianness=(int)1234, signed=(boolean)true, width=(int)16, depth=(int)16, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 2 ]");
+      ("audio/x-raw, format=(string)S16_LE, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 2 ]");
   c2 = gst_caps_from_string
-      ("audio/x-raw-int, width=(int)16, depth=(int)16, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 2 ], endianness=(int){ 1234, 4321 }, signed=(boolean){ true, false }; audio/x-raw-int, width=(int)16, depth=(int)16, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 11 ], endianness=(int){ 1234, 4321 }, signed=(boolean){ true, false }; audio/x-raw-int, width=(int)16, depth=(int)[ 1, 16 ], rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 11 ], endianness=(int){ 1234, 4321 }, signed=(boolean){ true, false }");
+      ("audio/x-raw, format=(string) { S16_LE, S16_BE, U16_LE, U16_BE }, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 2 ]; audio/x-raw, format=(string) { S16_LE, S16_BE, U16_LE, U16_BE }, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 11 ]; audio/x-raw, format=(string) { S16_LE, S16_BE, U16_LE, U16_BE }, rate=(int)[ 1, 2147483647 ], channels=(int)[ 1, 11 ]");
 
   test = gst_caps_intersect_full (c1, c2, GST_CAPS_INTERSECT_FIRST);
   fail_unless_equals_int (gst_caps_get_size (test), 1);
@@ -987,7 +915,6 @@ gst_caps_suite (void)
   tcase_add_test (tc_chain, test_from_string);
   tcase_add_test (tc_chain, test_double_append);
   tcase_add_test (tc_chain, test_mutability);
-  tcase_add_test (tc_chain, test_buffer);
   tcase_add_test (tc_chain, test_static_caps);
   tcase_add_test (tc_chain, test_simplify);
   tcase_add_test (tc_chain, test_truncate);
