@@ -39,12 +39,17 @@ GST_START_TEST (create_events)
   }
   /* FLUSH_STOP */
   {
-    event = gst_event_new_flush_stop ();
+    gboolean reset_time;
+
+    event = gst_event_new_flush_stop (TRUE);
     fail_if (event == NULL);
     fail_unless (GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_STOP);
     fail_unless (GST_EVENT_IS_UPSTREAM (event));
     fail_unless (GST_EVENT_IS_DOWNSTREAM (event));
     fail_unless (GST_EVENT_IS_SERIALIZED (event));
+
+    gst_event_parse_flush_stop (event, &reset_time);
+    fail_unless (reset_time == TRUE);
     gst_event_unref (event);
   }
   /* EOS */
@@ -57,63 +62,31 @@ GST_START_TEST (create_events)
     fail_unless (GST_EVENT_IS_SERIALIZED (event));
     gst_event_unref (event);
   }
-  /* NEWSEGMENT */
+  /* SEGMENT */
   {
-    gdouble rate, applied_rate;
-    GstFormat format;
-    gint64 start, end, base;
-    gboolean update;
+    GstSegment segment, parsed;
 
-    event =
-        gst_event_new_new_segment (FALSE, 0.5, GST_FORMAT_TIME, 1, G_MAXINT64,
-        0xdeadbeef);
+    gst_segment_init (&segment, GST_FORMAT_TIME);
+    segment.rate = 0.5;
+    segment.applied_rate = 1.0;
+    segment.start = 1;
+    segment.stop = G_MAXINT64;
+    segment.time = 0xdeadbeef;
+
+    event = gst_event_new_segment (&segment);
     fail_if (event == NULL);
-    fail_unless (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT);
+    fail_unless (GST_EVENT_TYPE (event) == GST_EVENT_SEGMENT);
     fail_if (GST_EVENT_IS_UPSTREAM (event));
     fail_unless (GST_EVENT_IS_DOWNSTREAM (event));
     fail_unless (GST_EVENT_IS_SERIALIZED (event));
 
-    gst_event_parse_new_segment (event, &update, &rate, &format, &start, &end,
-        &base);
-    fail_unless (update == FALSE);
-    fail_unless (rate == 0.5);
-    fail_unless (format == GST_FORMAT_TIME);
-    fail_unless (start == 1);
-    fail_unless (end == G_MAXINT64);
-    fail_unless (base == 0xdeadbeef);
-
-    /* Check that the new segment was created with applied_rate of 1.0 */
-    gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
-        &format, &start, &end, &base);
-
-    fail_unless (update == FALSE);
-    fail_unless (rate == 0.5);
-    fail_unless (applied_rate == 1.0);
-    fail_unless (format == GST_FORMAT_TIME);
-    fail_unless (start == 1);
-    fail_unless (end == G_MAXINT64);
-
-    gst_event_unref (event);
-
-    event =
-        gst_event_new_new_segment_full (TRUE, 0.75, 0.5, GST_FORMAT_BYTES, 0,
-        G_MAXINT64 - 1, 0xdeadbeef);
-
-    fail_if (event == NULL);
-    fail_unless (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT);
-    fail_if (GST_EVENT_IS_UPSTREAM (event));
-    fail_unless (GST_EVENT_IS_DOWNSTREAM (event));
-    fail_unless (GST_EVENT_IS_SERIALIZED (event));
-
-    gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
-        &format, &start, &end, &base);
-
-    fail_unless (update == TRUE);
-    fail_unless (rate == 0.75);
-    fail_unless (applied_rate == 0.5);
-    fail_unless (format == GST_FORMAT_BYTES);
-    fail_unless (start == 0);
-    fail_unless (end == (G_MAXINT64 - 1));
+    gst_event_copy_segment (event, &parsed);
+    fail_unless (parsed.rate == 0.5);
+    fail_unless (parsed.applied_rate == 1.0);
+    fail_unless (parsed.format == GST_FORMAT_TIME);
+    fail_unless (parsed.start == 1);
+    fail_unless (parsed.stop == G_MAXINT64);
+    fail_unless (parsed.time == 0xdeadbeef);
 
     gst_event_unref (event);
   }
@@ -143,32 +116,32 @@ GST_START_TEST (create_events)
     GstClockTimeDiff ctd1 = G_GINT64_CONSTANT (10), ctd2;
     GstClockTime ct1 = G_GUINT64_CONSTANT (20), ct2;
 
-    event = gst_event_new_qos (p1, ctd1, ct1);
+    event = gst_event_new_qos (t1, p1, ctd1, ct1);
     fail_if (event == NULL);
     fail_unless (GST_EVENT_TYPE (event) == GST_EVENT_QOS);
     fail_unless (GST_EVENT_IS_UPSTREAM (event));
     fail_if (GST_EVENT_IS_DOWNSTREAM (event));
     fail_if (GST_EVENT_IS_SERIALIZED (event));
 
-    gst_event_parse_qos (event, &p2, &ctd2, &ct2);
+    gst_event_parse_qos (event, &t2, &p2, &ctd2, &ct2);
     fail_unless (p1 == p2);
     fail_unless (ctd1 == ctd2);
     fail_unless (ct1 == ct2);
-    gst_event_parse_qos_full (event, &t2, &p2, &ctd2, &ct2);
-    fail_unless (t2 == GST_QOS_TYPE_UNDERFLOW);
+    gst_event_parse_qos (event, &t2, &p2, &ctd2, &ct2);
+    fail_unless (t2 == GST_QOS_TYPE_THROTTLE);
     fail_unless (p1 == p2);
     fail_unless (ctd1 == ctd2);
     fail_unless (ct1 == ct2);
     gst_event_unref (event);
 
     ctd1 = G_GINT64_CONSTANT (-10);
-    event = gst_event_new_qos (p1, ctd1, ct1);
-    gst_event_parse_qos_full (event, &t2, &p2, &ctd2, &ct2);
-    fail_unless (t2 == GST_QOS_TYPE_OVERFLOW);
+    event = gst_event_new_qos (t1, p1, ctd1, ct1);
+    gst_event_parse_qos (event, &t2, &p2, &ctd2, &ct2);
+    fail_unless (t2 == GST_QOS_TYPE_THROTTLE);
     gst_event_unref (event);
 
-    event = gst_event_new_qos_full (t1, p1, ctd1, ct1);
-    gst_event_parse_qos_full (event, &t2, &p2, &ctd2, &ct2);
+    event = gst_event_new_qos (t1, p1, ctd1, ct1);
+    gst_event_parse_qos (event, &t2, &p2, &ctd2, &ct2);
     fail_unless (t2 == GST_QOS_TYPE_THROTTLE);
     fail_unless (p1 == p2);
     fail_unless (ctd1 == ctd2);
@@ -296,14 +269,16 @@ static GTimeVal sent_event_time;
 static GstEvent *got_event_before_q, *got_event_after_q;
 static GTimeVal got_event_time;
 
-static gboolean
-event_probe (GstPad * pad, GstMiniObject ** data, gpointer user_data)
+static GstProbeReturn
+event_probe (GstPad * pad, GstProbeType type, gpointer type_data,
+    gpointer user_data)
 {
+  GstMiniObject *data = type_data;
   gboolean before_q = (gboolean) GPOINTER_TO_INT (user_data);
 
-  fail_unless (GST_IS_EVENT (data));
+  GST_DEBUG ("event probe called %p", data);
 
-  GST_DEBUG ("event probe called");
+  fail_unless (GST_IS_EVENT (data));
 
   if (before_q) {
     switch (GST_EVENT_TYPE (GST_EVENT (data))) {
@@ -336,7 +311,66 @@ event_probe (GstPad * pad, GstMiniObject ** data, gpointer user_data)
     }
   }
 
-  return TRUE;
+  return GST_PROBE_OK;
+}
+
+
+typedef struct
+{
+  GMutex *lock;
+  GCond *cond;
+  gboolean signaled;
+} SignalData;
+
+static void
+signal_data_init (SignalData * data)
+{
+  GST_DEBUG ("init %p", data);
+  data->lock = g_mutex_new ();
+  data->cond = g_cond_new ();
+  data->signaled = FALSE;
+}
+
+static void
+signal_data_cleanup (SignalData * data)
+{
+  GST_DEBUG ("free %p", data);
+  g_mutex_free (data->lock);
+  g_cond_free (data->cond);
+}
+
+static void
+signal_data_signal (SignalData * data)
+{
+  g_mutex_lock (data->lock);
+  data->signaled = TRUE;
+  g_cond_broadcast (data->cond);
+  GST_DEBUG ("signaling %p", data);
+  g_mutex_unlock (data->lock);
+}
+
+static void
+signal_data_wait (SignalData * data)
+{
+  g_mutex_lock (data->lock);
+  GST_DEBUG ("signal wait %p", data);
+  while (!data->signaled)
+    g_cond_wait (data->cond, data->lock);
+  GST_DEBUG ("signal wait done %p", data);
+  g_mutex_unlock (data->lock);
+}
+
+static GstProbeReturn
+signal_blocked (GstPad * pad, GstProbeType type, gpointer type_data,
+    gpointer user_data)
+{
+  SignalData *data = (SignalData *) user_data;
+
+  GST_DEBUG ("signal called %p", data);
+  signal_data_signal (data);
+  GST_DEBUG ("signal done %p", data);
+
+  return GST_PROBE_OK;
 }
 
 static void test_event
@@ -346,6 +380,8 @@ static void test_event
   GstEvent *event;
   GstPad *peer;
   gint i;
+  SignalData data;
+  gulong id;
 
   got_event_before_q = got_event_after_q = NULL;
 
@@ -361,15 +397,22 @@ static void test_event
   got_event_time.tv_sec = 0;
   got_event_time.tv_usec = 0;
 
+  signal_data_init (&data);
+
   /* We block the pad so the stream lock is released and we can send the event */
-  fail_unless (gst_pad_set_blocked (fake_srcpad, TRUE) == TRUE);
+  id = gst_pad_add_probe (fake_srcpad, GST_PROBE_TYPE_BLOCK,
+      signal_blocked, &data, NULL);
+  fail_unless (id != 0);
+
+  signal_data_wait (&data);
 
   /* We send on the peer pad, since the pad is blocked */
+  GST_DEBUG ("sending event %p", event);
   fail_unless ((peer = gst_pad_get_peer (pad)) != NULL);
   gst_pad_send_event (peer, event);
   gst_object_unref (peer);
 
-  fail_unless (gst_pad_set_blocked (fake_srcpad, FALSE) == TRUE);
+  gst_pad_remove_probe (fake_srcpad, id);
 
   if (expect_before_q) {
     /* Wait up to 5 seconds for the event to appear */
@@ -404,6 +447,8 @@ static void test_event
     gst_event_unref (got_event_after_q);
 
   got_event_before_q = got_event_after_q = NULL;
+
+  signal_data_cleanup (&data);
 }
 
 static gint64
@@ -440,12 +485,12 @@ GST_START_TEST (send_custom_events)
 
   /* add pad-probes to faksrc.src and fakesink.sink */
   fail_if ((srcpad = gst_element_get_static_pad (fakesrc, "src")) == NULL);
-  gst_pad_add_event_probe (srcpad, (GCallback) event_probe,
-      GINT_TO_POINTER (TRUE));
+  gst_pad_add_probe (srcpad, GST_PROBE_TYPE_EVENT,
+      event_probe, GINT_TO_POINTER (TRUE), NULL);
 
   fail_if ((sinkpad = gst_element_get_static_pad (fakesink, "sink")) == NULL);
-  gst_pad_add_event_probe (sinkpad, (GCallback) event_probe,
-      GINT_TO_POINTER (FALSE));
+  gst_pad_add_probe (sinkpad, GST_PROBE_TYPE_EVENT,
+      event_probe, GINT_TO_POINTER (FALSE), NULL);
 
   /* Upstream events */
   test_event (pipeline, GST_EVENT_CUSTOM_UPSTREAM, sinkpad, TRUE, srcpad);
