@@ -27,8 +27,9 @@
 #include <gst/check/gstcheck.h>
 #include <gst/base/gstbasesrc.h>
 
-static gboolean
-eos_event_counter (GstObject * pad, GstEvent * event, guint * p_num_eos)
+static GstProbeReturn
+eos_event_counter (GstObject * pad, GstProbeType type, GstEvent * event,
+    guint * p_num_eos)
 {
   fail_unless (event != NULL);
   fail_unless (GST_IS_EVENT (event));
@@ -36,7 +37,7 @@ eos_event_counter (GstObject * pad, GstEvent * event, guint * p_num_eos)
   if (GST_EVENT_TYPE (event) == GST_EVENT_EOS)
     *p_num_eos += 1;
 
-  return TRUE;
+  return GST_PROBE_OK;
 }
 
 /* basesrc_eos_events_push_live_op:
@@ -76,8 +77,8 @@ GST_START_TEST (basesrc_eos_events_push_live_op)
   srcpad = gst_element_get_static_pad (src, "src");
   fail_unless (srcpad != NULL);
 
-  probe = gst_pad_add_event_probe (srcpad,
-      G_CALLBACK (eos_event_counter), &num_eos);
+  probe = gst_pad_add_probe (srcpad, GST_PROBE_TYPE_EVENT,
+      (GstPadProbeCallback) eos_event_counter, &num_eos, NULL);
 
   bus = gst_element_get_bus (pipe);
 
@@ -88,12 +89,8 @@ GST_START_TEST (basesrc_eos_events_push_live_op)
   /* wait a second, then do controlled shutdown */
   g_usleep (GST_USECOND * 1);
 
-  /* shut down source only (should send EOS event) ... */
-  gst_element_set_state (src, GST_STATE_NULL);
-  state_ret = gst_element_get_state (src, NULL, NULL, -1);
-  fail_unless (state_ret == GST_STATE_CHANGE_SUCCESS);
-
-  fail_unless (gst_element_set_locked_state (src, TRUE) == TRUE);
+  /* shut down pipeline (should send EOS message) ... */
+  gst_element_send_event (pipe, gst_event_new_eos ());
 
   /* ... and wait for the EOS message from the sink */
   msg = gst_bus_poll (bus, GST_MESSAGE_EOS | GST_MESSAGE_ERROR, -1);
@@ -110,7 +107,7 @@ GST_START_TEST (basesrc_eos_events_push_live_op)
   /* make sure source hasn't sent a second one when going PAUSED => READY */
   fail_unless (num_eos == 1);
 
-  gst_pad_remove_event_probe (srcpad, probe);
+  gst_pad_remove_probe (srcpad, probe);
   gst_object_unref (srcpad);
   gst_message_unref (msg);
   gst_object_unref (bus);
@@ -119,6 +116,8 @@ GST_START_TEST (basesrc_eos_events_push_live_op)
 
 GST_END_TEST;
 
+
+
 /* basesrc_eos_events_push:
  *  - make sure source only sends one EOS when operating in push-mode,
  *    reaching the max number of buffers, and is then shut down.
@@ -156,8 +155,8 @@ GST_START_TEST (basesrc_eos_events_push)
   srcpad = gst_element_get_static_pad (src, "src");
   fail_unless (srcpad != NULL);
 
-  probe = gst_pad_add_event_probe (srcpad,
-      G_CALLBACK (eos_event_counter), &num_eos);
+  probe = gst_pad_add_probe (srcpad, GST_PROBE_TYPE_EVENT,
+      (GstPadProbeCallback) eos_event_counter, &num_eos, NULL);
 
   bus = gst_element_get_bus (pipe);
 
@@ -179,7 +178,7 @@ GST_START_TEST (basesrc_eos_events_push)
   /* make sure source hasn't sent a second one when going PAUSED => READY */
   fail_unless (num_eos == 1);
 
-  gst_pad_remove_event_probe (srcpad, probe);
+  gst_pad_remove_probe (srcpad, probe);
   gst_object_unref (srcpad);
   gst_message_unref (msg);
   gst_object_unref (bus);
@@ -223,8 +222,8 @@ GST_START_TEST (basesrc_eos_events_pull_live_op)
   srcpad = gst_element_get_static_pad (src, "src");
   fail_unless (srcpad != NULL);
 
-  probe = gst_pad_add_event_probe (srcpad,
-      G_CALLBACK (eos_event_counter), &num_eos);
+  probe = gst_pad_add_probe (srcpad, GST_PROBE_TYPE_EVENT,
+      (GstPadProbeCallback) eos_event_counter, &num_eos, NULL);
 
   gst_element_set_state (pipe, GST_STATE_PLAYING);
   state_ret = gst_element_get_state (pipe, NULL, NULL, -1);
@@ -249,7 +248,7 @@ GST_START_TEST (basesrc_eos_events_pull_live_op)
   /* make sure source hasn't sent an EOS when going PAUSED => READY either */
   fail_unless (num_eos == 0);
 
-  gst_pad_remove_event_probe (srcpad, probe);
+  gst_pad_remove_probe (srcpad, probe);
   gst_object_unref (srcpad);
   gst_object_unref (pipe);
 }
@@ -295,8 +294,8 @@ GST_START_TEST (basesrc_eos_events_pull)
   srcpad = gst_element_get_static_pad (src, "src");
   fail_unless (srcpad != NULL);
 
-  probe = gst_pad_add_event_probe (srcpad,
-      G_CALLBACK (eos_event_counter), &num_eos);
+  probe = gst_pad_add_probe (srcpad, GST_PROBE_TYPE_EVENT,
+      (GstPadProbeCallback) eos_event_counter, &num_eos, NULL);
 
   bus = gst_element_get_bus (pipe);
 
@@ -318,7 +317,7 @@ GST_START_TEST (basesrc_eos_events_pull)
   /* make sure source hasn't sent an EOS when going PAUSED => READY either */
   fail_unless (num_eos == 0);
 
-  gst_pad_remove_event_probe (srcpad, probe);
+  gst_pad_remove_probe (srcpad, probe);
   gst_object_unref (srcpad);
   gst_message_unref (msg);
   gst_object_unref (bus);
@@ -365,8 +364,8 @@ GST_START_TEST (basesrc_eos_events_push_live_eos)
   srcpad = gst_element_get_static_pad (src, "src");
   fail_unless (srcpad != NULL);
 
-  probe = gst_pad_add_event_probe (srcpad,
-      G_CALLBACK (eos_event_counter), &num_eos);
+  probe = gst_pad_add_probe (srcpad, GST_PROBE_TYPE_EVENT,
+      (GstPadProbeCallback) eos_event_counter, &num_eos, NULL);
 
   bus = gst_element_get_bus (pipe);
 
@@ -396,7 +395,7 @@ GST_START_TEST (basesrc_eos_events_push_live_eos)
   /* make sure source hasn't sent a second one when going PAUSED => READY */
   fail_unless (num_eos == 1);
 
-  gst_pad_remove_event_probe (srcpad, probe);
+  gst_pad_remove_probe (srcpad, probe);
   gst_object_unref (srcpad);
   gst_message_unref (msg);
   gst_object_unref (bus);
@@ -442,8 +441,8 @@ GST_START_TEST (basesrc_eos_events_pull_live_eos)
   srcpad = gst_element_get_static_pad (src, "src");
   fail_unless (srcpad != NULL);
 
-  probe = gst_pad_add_event_probe (srcpad,
-      G_CALLBACK (eos_event_counter), &num_eos);
+  probe = gst_pad_add_probe (srcpad, GST_PROBE_TYPE_EVENT,
+      (GstPadProbeCallback) eos_event_counter, &num_eos, NULL);
 
   bus = gst_element_get_bus (pipe);
 
@@ -473,7 +472,7 @@ GST_START_TEST (basesrc_eos_events_pull_live_eos)
   /* make sure source hasn't sent a second one when going PAUSED => READY */
   fail_unless (num_eos == 0);
 
-  gst_pad_remove_event_probe (srcpad, probe);
+  gst_pad_remove_probe (srcpad, probe);
   gst_object_unref (srcpad);
   gst_message_unref (msg);
   gst_object_unref (bus);
@@ -483,8 +482,8 @@ GST_START_TEST (basesrc_eos_events_pull_live_eos)
 GST_END_TEST;
 
 
-static gboolean
-newsegment_event_catcher (GstObject * pad, GstEvent * event,
+static GstProbeReturn
+segment_event_catcher (GstObject * pad, GstProbeType type, GstEvent * event,
     gpointer * user_data)
 {
   GstEvent **last_event = (GstEvent **) user_data;
@@ -492,17 +491,17 @@ newsegment_event_catcher (GstObject * pad, GstEvent * event,
   fail_unless (GST_IS_EVENT (event));
   fail_unless (user_data != NULL);
 
-  if (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT) {
+  if (GST_EVENT_TYPE (event) == GST_EVENT_SEGMENT) {
     if (*last_event)
       gst_event_unref (*last_event);
     *last_event = gst_event_copy (event);
   }
 
-  return TRUE;
+  return GST_PROBE_OK;
 }
 
 /* basesrc_seek_events_rate_update:
- *  - make sure we get expected newsegment after sending a seek event
+ *  - make sure we get expected segment after sending a seek event
  */
 GST_START_TEST (basesrc_seek_events_rate_update)
 {
@@ -512,10 +511,10 @@ GST_START_TEST (basesrc_seek_events_rate_update)
   GstBus *bus;
   GstPad *probe_pad;
   guint probe;
-  GstEvent *newseg_event = NULL;
+  GstEvent *seg_event = NULL;
   GstEvent *rate_seek;
   gboolean event_ret;
-  gdouble rate = 0.5;
+  const GstSegment *segment;
 
   pipe = gst_pipeline_new ("pipeline");
   sink = gst_element_factory_make ("fakesink", "sink");
@@ -536,8 +535,8 @@ GST_START_TEST (basesrc_seek_events_rate_update)
   probe_pad = gst_element_get_static_pad (sink, "sink");
   fail_unless (probe_pad != NULL);
 
-  probe = gst_pad_add_event_probe (probe_pad,
-      G_CALLBACK (newsegment_event_catcher), &newseg_event);
+  probe = gst_pad_add_probe (probe_pad, GST_PROBE_TYPE_EVENT,
+      (GstPadProbeCallback) segment_event_catcher, &seg_event, NULL);
 
   /* prepare the seek */
   rate_seek = gst_event_new_seek (0.5, GST_FORMAT_TIME, GST_SEEK_FLAG_NONE,
@@ -560,12 +559,8 @@ GST_START_TEST (basesrc_seek_events_rate_update)
   /* wait a second, then do controlled shutdown */
   g_usleep (GST_USECOND * 1);
 
-  /* shut down source only (should send EOS event) ... */
-  gst_element_set_state (src, GST_STATE_NULL);
-  state_ret = gst_element_get_state (src, NULL, NULL, -1);
-  fail_unless (state_ret == GST_STATE_CHANGE_SUCCESS);
-
-  fail_unless (gst_element_set_locked_state (src, TRUE) == TRUE);
+  /* shut down pipeline only (should send EOS message) ... */
+  gst_element_send_event (pipe, gst_event_new_eos ());
 
   /* ... and wait for the EOS message from the sink */
   msg = gst_bus_poll (bus, GST_MESSAGE_EOS | GST_MESSAGE_ERROR, -1);
@@ -579,16 +574,15 @@ GST_START_TEST (basesrc_seek_events_rate_update)
   GST_INFO ("stopped");
 
   /* check that we have go the event */
-  fail_unless (newseg_event != NULL);
+  fail_unless (seg_event != NULL);
 
-  gst_event_parse_new_segment (newseg_event, NULL, &rate, NULL, NULL, NULL,
-      NULL);
-  fail_unless (rate == 0.5);
+  gst_event_parse_segment (seg_event, &segment);
+  fail_unless (segment->rate == 0.5);
 
-  gst_pad_remove_event_probe (probe_pad, probe);
+  gst_pad_remove_probe (probe_pad, probe);
   gst_object_unref (probe_pad);
   gst_message_unref (msg);
-  gst_event_unref (newseg_event);
+  gst_event_unref (seg_event);
   gst_object_unref (bus);
   gst_object_unref (pipe);
 }
